@@ -23,22 +23,21 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.c,v 1.5 2003/05/19 04:56:58 fuyu Exp $
+ * $Id: bbs_thread_view.c,v 1.8 2003/06/06 19:37:23 fuyu Exp $
  */
 
 #include "config.h"
 
+#include "ochusha_private.h"
 #include "ochusha.h"
-#include "ochusha_ui.h"
-
 #include "ochusha_bbs_thread.h"
-
-#include "marshal.h"
-
 #include "worker.h"
 
+#include "ochusha_ui.h"
 #include "bbs_thread_view.h"
 
+#include "marshal.h"
+
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -61,11 +60,17 @@ static gboolean text_view_get_iter_at_link_tag_from_event(
 						GtkTextView *text_view,
 						GtkTextIter *iter,
 						GdkEventAny *event);
+static gboolean enter_notify_event_cb(GtkWidget *widget,
+				      GdkEventCrossing *event,
+				      BBSThreadView *view);
+static gboolean leave_notify_event_cb(GtkWidget *widget,
+				      GdkEventCrossing *event,
+				      BBSThreadView *view);
 static gboolean motion_notify_event_cb(GtkTextView *text_view,
 				       GdkEventMotion *event,
 				       BBSThreadView *bbs_thread_view);
 static void emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event);
-static void emit_link_mouse_out(BBSThreadView *view, GdkEventMotion *event);
+static void emit_link_mouse_out(BBSThreadView *view, GdkEventAny *event);
 
 static gboolean button_press_event_cb(GtkTextView *text_view,
 				      GdkEventButton *event,
@@ -109,6 +114,8 @@ enum {
   LINK_MOUSE_OUT_SIGNAL,
   LINK_MOUSE_PRESS_SIGNAL,
   LINK_MOUSE_RELEASE_SIGNAL,
+  VIEW_ENTER_NOTIFY_SIGNAL,
+  VIEW_LEAVE_NOTIFY_SIGNAL,
   SCROLL_TO_SIGNAL,
   LAST_SIGNAL
 };
@@ -121,7 +128,7 @@ enum {
 
 
 static GtkScrolledWindowClass *parent_class = NULL;
-static gint bbs_thread_view_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0 };
+static gint bbs_thread_view_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0, 0, 0 };
 static GtkTextTagTable *default_tag_table;
 
 static GtkTextTag *link_tags[2];
@@ -274,6 +281,25 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 		 G_TYPE_POINTER,
 		 G_TYPE_STRING);
 
+  bbs_thread_view_signals[VIEW_ENTER_NOTIFY_SIGNAL] =
+    g_signal_new("view_enter_notify",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(BBSThreadViewClass, view_enter_notify),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__BOXED,
+		 G_TYPE_NONE, 1,
+		 GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
+  bbs_thread_view_signals[VIEW_LEAVE_NOTIFY_SIGNAL] =
+    g_signal_new("view_leave_notify",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(BBSThreadViewClass, view_leave_notify),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__BOXED,
+		 G_TYPE_NONE, 1,
+		 GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
+
   bbs_thread_view_signals[SCROLL_TO_SIGNAL] =
     g_signal_new("scroll_to",
 		 G_TYPE_FROM_CLASS(klass),
@@ -306,6 +332,18 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 }
 
 
+#if TRACE_MEMORY_USAGE
+static void
+trace_free(gpointer pointer)
+{
+  G_FREE(pointer);
+}
+#define TRACE_FREE	trace_free
+#else
+#define TRACE_FREE	g_free
+#endif
+
+
 static void
 bbs_thread_view_init(BBSThreadView *view)
 {
@@ -335,6 +373,10 @@ bbs_thread_view_init(BBSThreadView *view)
 
   g_signal_connect(G_OBJECT(widget), "realize",
 		   G_CALLBACK(text_view_realize_cb), view);
+  g_signal_connect(G_OBJECT(widget), "enter_notify_event",
+		   G_CALLBACK(enter_notify_event_cb), view);
+  g_signal_connect(G_OBJECT(widget), "leave_notify_event",
+		   G_CALLBACK(leave_notify_event_cb), view);
   g_signal_connect(G_OBJECT(widget), "motion_notify_event",
 		   G_CALLBACK(motion_notify_event_cb), view);
   g_signal_connect(G_OBJECT(widget), "button_press_event",
@@ -348,7 +390,7 @@ bbs_thread_view_init(BBSThreadView *view)
 
   view->last_offset = -1;
   view->thread = NULL;
-  view->link_table = g_hash_table_new_full(NULL, NULL, NULL, g_free);
+  view->link_table = g_hash_table_new_full(NULL, NULL, NULL, TRACE_FREE);
 
   view->insert_mark = gtk_text_buffer_get_mark(view->text_buffer, "insert");
   view->selection_bound_mark = gtk_text_buffer_get_mark(view->text_buffer,
@@ -404,7 +446,7 @@ bbs_thread_view_destroy(GtkObject *object)
 
   if (view->search_key != NULL)
     {
-      g_free(view->search_key);
+      G_FREE(view->search_key);
       view->search_key = NULL;
     }
 
@@ -429,8 +471,9 @@ text_view_realize_cb(GtkTextView *text_view, BBSThreadView *bbs_thread_view)
   GdkWindow *window = gtk_text_view_get_window(text_view,
 					       GTK_TEXT_WINDOW_TEXT);
   gdk_window_set_events(window,
-			gdk_window_get_events(window)
-			& ~GDK_POINTER_MOTION_HINT_MASK);
+			(gdk_window_get_events(window)
+			 & ~GDK_POINTER_MOTION_HINT_MASK)
+			| GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
 
   if (bbs_thread_view->ibeam_cursor != NULL)
     gdk_cursor_unref(bbs_thread_view->ibeam_cursor);
@@ -500,6 +543,46 @@ text_view_get_iter_at_link_tag_from_event(GtkTextView *text_view,
 }
 
 
+static gboolean
+enter_notify_event_cb(GtkWidget *widget, GdkEventCrossing *event,
+		      BBSThreadView *view)
+{
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(view), FALSE);
+
+#if DEBUG_WIDGET_MOST
+  fprintf(stderr, "bbs_thread_view.c: enter_notify_event_cb\n");
+#endif
+
+  g_signal_emit(G_OBJECT(view),
+		bbs_thread_view_signals[VIEW_ENTER_NOTIFY_SIGNAL],
+		0,
+		event);
+
+  return FALSE;
+}
+
+
+static gboolean
+leave_notify_event_cb(GtkWidget *widget, GdkEventCrossing *event,
+		      BBSThreadView *view)
+{
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(view), FALSE);
+
+#if DEBUG_WIDGET_MOST
+  fprintf(stderr, "bbs_thread_view.c: leave_notify_event_cb\n");
+#endif
+  if (view->last_offset != -1)
+    emit_link_mouse_out(view, (GdkEventAny *)event);
+
+  g_signal_emit(G_OBJECT(view),
+		bbs_thread_view_signals[VIEW_LEAVE_NOTIFY_SIGNAL],
+		0,
+		event);
+
+  return FALSE;
+}
+
+
 static gboolean
 motion_notify_event_cb(GtkTextView *text_view, GdkEventMotion *event,
 		       BBSThreadView *bbs_thread_view)
@@ -517,13 +600,13 @@ motion_notify_event_cb(GtkTextView *text_view, GdkEventMotion *event,
 	return FALSE;
 
       if (bbs_thread_view->last_offset != -1)
-	emit_link_mouse_out(bbs_thread_view, event);
+	emit_link_mouse_out(bbs_thread_view, (GdkEventAny *)event);
 
       bbs_thread_view->last_offset = start_offset;
       emit_link_mouse_over(bbs_thread_view, event);
     }
   else if (bbs_thread_view->last_offset != -1)
-    emit_link_mouse_out(bbs_thread_view, event);
+    emit_link_mouse_out(bbs_thread_view, (GdkEventAny *)event);
 
   return FALSE;
 }
@@ -559,7 +642,7 @@ emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event)
 
 
 static void
-emit_link_mouse_out(BBSThreadView *view, GdkEventMotion *event)
+emit_link_mouse_out(BBSThreadView *view, GdkEventAny *event)
 {
   gchar *link;
   if (view->last_offset == -1)
@@ -720,7 +803,7 @@ bbs_thread_view_clear(BBSThreadView *view)
   view->link_table = NULL;
   if (link_table != NULL)
     g_hash_table_destroy(link_table);
-  view->link_table = g_hash_table_new_full(NULL, NULL, NULL, g_free);
+  view->link_table = g_hash_table_new_full(NULL, NULL, NULL, TRACE_FREE);
 }
 
 
@@ -834,7 +917,7 @@ bbs_thread_view_append_text_as_link(BBSThreadView *view,
   gtk_text_buffer_apply_tag(view->text_buffer, get_alt_link_tag(view),
 			    &start, &view->iter);
   g_hash_table_insert(view->link_table,
-		      (gpointer)start_offset, g_strdup(link));
+		      (gpointer)start_offset, G_STRDUP(link));
 }
 
 
@@ -1093,7 +1176,7 @@ bbs_thread_view_find_forward_first_match(BBSThreadView *view,
   if (!view->use_regexp && !view->match_case)
     {
       gchar *tmp_text = g_utf8_casefold(text, -1);
-      g_free(text);
+      G_FREE(text);
       text = tmp_text;
     }
 
@@ -1131,7 +1214,7 @@ bbs_thread_view_find_forward_first_match(BBSThreadView *view,
     }
 
   if (text != NULL)
-    g_free(text);
+    G_FREE(text);
 
   return result;
 }
@@ -1160,7 +1243,7 @@ bbs_thread_view_find_backward_first_match(BBSThreadView *view,
   if (!view->use_regexp && !view->match_case)
     {
       gchar *tmp_text = g_utf8_casefold(text, -1);
-      g_free(text);
+      G_FREE(text);
       text = tmp_text;
     }
 
@@ -1215,7 +1298,7 @@ bbs_thread_view_find_backward_first_match(BBSThreadView *view,
     }
 
   if (text != NULL)
-    g_free(text);
+    G_FREE(text);
 
   return result;
 }
@@ -1368,7 +1451,7 @@ bbs_thread_view_find(BBSThreadView *view, const gchar *key,
 
   if (view->search_key != NULL)
     {
-      g_free(view->search_key);
+      G_FREE(view->search_key);
       view->search_key = NULL;
     }
 
@@ -1396,16 +1479,32 @@ bbs_thread_view_find(BBSThreadView *view, const gchar *key,
   else
     {
       if (match_case)
-	view->search_key = g_strdup(key);
+	view->search_key = G_STRDUP(key);
       else
+#if TRACE_MEMORY_USAGE
+	{
+	  gchar *tmp_key = g_utf8_casefold(key, -1);
+	  view->search_key = G_STRDUP(tmp_key);
+	  g_free(tmp_key);
+	}
+#else
 	view->search_key = g_utf8_casefold(key, -1);
+#endif
     }
   view->use_regexp = use_regexp;
 #else
   if (match_case)
-    view->search_key = g_strdup(key);
+    view->search_key = G_STRDUP(key);
   else
+#if TRACE_MEMORY_USAGE
+    {
+      gchar *tmp_key = g_utf8_casefold(key, -1);
+      view->search_key = G_STRDUP(tmp_key);
+      g_free(tmp_key);
+    }
+#else
     view->search_key = g_utf8_casefold(key, -1);
+#endif
   view->use_regexp = FALSE;
   view->regexp_available = FALSE;
 #endif
@@ -1451,7 +1550,7 @@ bbs_thread_view_invalidate_search_result(BBSThreadView *view)
 
   if (view->search_key != NULL)
     {
-      g_free(view->search_key);
+      G_FREE(view->search_key);
       view->search_key = NULL;
     }
 
