@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.h,v 1.3 2003/05/19 04:15:24 fuyu Exp $
+ * $Id: bbs_thread_view.h,v 1.5 2003/06/03 12:14:21 fuyu Exp $
  */
 
 #ifndef _BBS_THREAD_VIEW_H_
@@ -115,14 +115,19 @@ struct _BBSThreadViewClass
 			  GdkEventMotion *event,
 			  OchushaBBSThread *thread, const gchar *link);
   void (*link_mouse_out)(BBSThreadView *bbs_thread_view,
-			 GdkEventMotion *event,
+			 GdkEventAny *event,
 			 OchushaBBSThread *thread, const gchar *link);
-  void (*link_mouse_press)(BBSThreadView *bbs_therad_view,
-			   GdkEventButton *event,
-			   OchushaBBSThread *thread, const gchar *link);
-  void (*link_mouse_release)(BBSThreadView *bbs_thread_view,
-			     GdkEventButton *event,
-			     OchushaBBSThread *thread, const gchar *link);
+  gboolean (*link_mouse_press)(BBSThreadView *bbs_therad_view,
+			       GdkEventButton *event,
+			       OchushaBBSThread *thread, const gchar *link);
+  gboolean (*link_mouse_release)(BBSThreadView *bbs_thread_view,
+				 GdkEventButton *event,
+				 OchushaBBSThread *thread, const gchar *link);
+
+  void (*view_enter_notify)(BBSThreadView *bbs_thread_view,
+			    GdkEventCrossing *event);
+  void (*view_leave_notify)(BBSThreadView *bbs_thread_view,
+			    GdkEventCrossing *event);
 
   void (*scroll_to)(BBSThreadView *bbs_thread_view, gint to_where);
 };
