@@ -23,11 +23,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: board_properties.c,v 1.2 2003/05/14 20:58:12 fuyu Exp $
+ * $Id: board_properties.c,v 1.3 2003/06/06 19:37:23 fuyu Exp $
  */
 
-#include "ochusha_ui.h"
+#include "config.h"
 
+#include "ochusha_private.h"
+
+#include "ochusha_ui.h"
 #include "bulletin_board_ui.h"
 #include "board_properties.h"
 #include "boardlist_ui.h"
@@ -119,13 +122,7 @@ open_board_properties(OchushaApplication *application,
       return;
     }
 
-  states = (DialogStates *)calloc(1, sizeof(DialogStates));
-  if (states == NULL)
-    {
-      BOARD_PROPERTIES_UNLOCK;
-      return;
-    }
-
+  states = G_NEW0(DialogStates, 1);
   states->application = application;
   states->board = board;
   states->info = info;
@@ -245,8 +242,8 @@ apply_current_states(GtkWidget *dialog, DialogStates *states)
 
   if (gtk_toggle_button_get_active(states->use_as_default_button))
     {
-      free(states->application->user_default_filter.rule);
-      states->application->user_default_filter.rule = g_strdup(filter->rule);
+      G_FREE(states->application->user_default_filter.rule);
+      states->application->user_default_filter.rule = G_STRDUP(filter->rule);
       states->application->user_default_filter.ignore_threshold
 	= filter->ignore_threshold;
 
@@ -254,7 +251,7 @@ apply_current_states(GtkWidget *dialog, DialogStates *states)
 	{
 	  gchar *tmp = states->info->filter.rule;
 	  states->info->filter.rule = NULL;
-	  free(tmp);
+	  G_FREE(tmp);
 	}
 
       states->info->filter.ignore_threshold = filter->ignore_threshold;
@@ -269,12 +266,12 @@ apply_current_states(GtkWidget *dialog, DialogStates *states)
 	{
 	  gchar *tmp = states->info->filter.rule;
 	  states->info->filter.rule = NULL;
-	  free(tmp);
+	  G_FREE(tmp);
 	}
 
       if (!states->use_default)
 	{
-	  states->info->filter.rule = g_strdup(filter->rule);
+	  states->info->filter.rule = G_STRDUP(filter->rule);
 	  states->info->filter.ignore_threshold = filter->ignore_threshold;
 	}
 
