@@ -23,16 +23,17 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.10 2003/05/19 04:14:55 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.14 2003/06/07 14:35:30 fuyu Exp $
  */
 
 #include "config.h"
 
+#include "ochusha_private.h"
 #include "ochusha.h"
-#include "ochusha_ui.h"
-
 #include "ochusha_utils_2ch.h"
+#include "worker.h"
 
+#include "ochusha_ui.h"
 #include "bbs_thread_ui.h"
 #include "board_properties.h"
 #include "boardlist_ui.h"
@@ -42,8 +43,6 @@
 #include "text_search_window.h"
 #include "ugly_gtk2utils.h"
 
-#include "worker.h"
-
 #include <glib.h>
 #include <gtk/gtk.h>
 
@@ -180,6 +179,11 @@ initialize_boardlist(OchushaApplication *application)
 		   G_CALLBACK(category_expanded_cb), application);
   g_signal_connect(G_OBJECT(application->boardlist_view), "category_collapsed",
 		   G_CALLBACK(category_collapsed_cb), application);
+#if TRACE_MEMORY_USAGE
+  g_signal_connect(G_OBJECT(application->boardlist_view), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed),
+		   &application->boardlist_view);
+#endif
   gtk_widget_show(application->boardlist_view);
 
   application->contents_window
@@ -221,6 +225,11 @@ initialize_boardlist(OchushaApplication *application)
 			     "apply_filter_to_the_current_threadlist");
   g_signal_connect(G_OBJECT(filter_button), "toggled",
 		   G_CALLBACK(filter_toggled_cb), application);
+#if TRACE_MEMORY_USAGE
+  g_signal_connect(G_OBJECT(filter_button), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed),
+		   &application->filter_button);
+#endif
 
   gtk_toolbar_prepend_space(toolbar);
   gtk_toolbar_insert_stock(toolbar, GTK_STOCK_REFRESH,
@@ -277,7 +286,7 @@ initialize_boardlist(OchushaApplication *application)
 		   application);
 
   if (ochusha_bbs_table_read_boardlist_xml(application->table,
-					   &application->conf))
+					   &application->config))
     {
       setup_boardlist(application);
       return;
@@ -287,6 +296,18 @@ initialize_boardlist(OchushaApplication *application)
 }
 
 
+#if TRACE_MEMORY_USAGE
+static void
+trace_free(gpointer pointer)
+{
+  G_FREE(pointer);
+}
+#define TRACE_FREE	trace_free
+#else
+#define TRACE_FREE	g_free
+#endif
+
+
 static BoardCategoryGUIInfo *
 ensure_board_category_info(OchushaBoardCategory *category)
 {
@@ -294,9 +315,9 @@ ensure_board_category_info(OchushaBoardCategory *category)
 						  category_gui_info_id);
   if (info == NULL)
     {
-      info = g_new0(BoardCategoryGUIInfo, 1);
+      info = G_NEW0(BoardCategoryGUIInfo, 1);
       g_object_set_qdata_full(G_OBJECT(category), category_gui_info_id,
-			      info, (GDestroyNotify)g_free);
+			      info, (GDestroyNotify)TRACE_FREE);
     }
 
   return info;
@@ -306,12 +327,8 @@ ensure_board_category_info(OchushaBoardCategory *category)
 void
 refresh_boardlist(OchushaApplication *application)
 {
-  WorkerJob *job = (WorkerJob *)calloc(1, sizeof(WorkerJob));
-  if (job == NULL)
-    {
-      fprintf(stderr, "Couldn't start boardlist reader.\n");
-      return;
-    }
+  WorkerJob *job = G_NEW0(WorkerJob, 1);
+
   job->canceled = FALSE;
   job->job = build_boardlist;
   job->args = application;
@@ -339,7 +356,7 @@ finalize_boardlist(OchushaApplication *application)
   if (application->table != NULL)
     {
       if (!ochusha_bbs_table_write_boardlist_xml(application->table,
-						 &application->conf))
+						 &application->config))
 	fprintf(stderr, "Couldn't write boardlist2.xml.\n");
     }
 
@@ -530,14 +547,19 @@ text_search_window_query_changed_cb(TextSearchWindow *window, const gchar *key,
   gchar *normalized_key;
 
   if (search_query_data.key != NULL)
-    g_free(search_query_data.key);
+    G_FREE(search_query_data.key);
 
   normalized_key = g_utf8_normalize(key, -1, G_NORMALIZE_ALL);
   if (!match_case)
     {
       gchar *case_normalized_key = g_utf8_casefold(normalized_key, -1);
-      g_free(normalized_key);
+      G_FREE(normalized_key);
+#if TRACE_MEMORY_USAGE
+      normalized_key = G_STRDUP(case_normalized_key);
+      g_free(case_normalized_key);
+#else
       normalized_key = case_normalized_key;
+#endif
     }
   search_query_data.key = normalized_key;
   search_query_data.direction = direction;
@@ -590,7 +612,7 @@ setup_threadlist_search_window(OchushaApplication *application)
 {
   if (search_query_data.key != NULL)
     {
-      g_free(search_query_data.key);
+      G_FREE(search_query_data.key);
       search_query_data.key = NULL;
     }
 
@@ -607,6 +629,11 @@ setup_threadlist_search_window(OchushaApplication *application)
   g_signal_connect(application->threadlist_search_window, "find_next",
 		   G_CALLBACK(text_search_window_find_next_cb),
 		   application);
+#if TRACE_MEMORY_USAGE
+  g_signal_connect(application->threadlist_search_window, "destroy",
+		   G_CALLBACK(gtk_widget_destroyed),
+		   &application->threadlist_search_window);
+#endif
   gtk_window_set_title(GTK_WINDOW(application->threadlist_search_window),
 		       _("Find in the current threadlist"));
   gtk_window_set_transient_for(
@@ -635,6 +662,7 @@ advance_threadlist_search(OchushaApplication *application,
     default:
       /* SHOULD NOT reached here */
       /* THREADLIST_SEARCH_ACTION_TERMINATEcaller¦ǽ */
+      break;
     }
 
   if (application->threadlist_search_window == NULL
@@ -766,8 +794,8 @@ static void
 select_category_cb(BoardlistView *view, OchushaBoardCategory *category,
 		   gpointer user_data)
 {
-  fprintf(stderr, "select_category_cb: 0x%x, 0x%x, 0x%x\n",
-	  (int)view, (int)category, (int)user_data);
+  fprintf(stderr, "select_category_cb: %p, %p, %p\n",
+	  view, category, user_data);
 }
 #endif
 
@@ -953,8 +981,6 @@ boardlist_read_board_element_cb(OchushaBBSTable *table,
 				OchushaApplication *application)
 {
   BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
-  info->last_modified = ochusha_utils_get_attribute_string(board_attributes,
-							   "last_modified");
   info->enable_filter = ochusha_utils_get_attribute_boolean(board_attributes,
 							    "enable_filter");
   info->filter.rule = ochusha_utils_get_attribute_string(board_attributes,
@@ -1015,9 +1041,6 @@ boardlist_write_board_element_cb(OchushaBBSTable *table,
   BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
   OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(boardlist_xml, info, enable_filter);
 
-  if (info->last_modified != NULL)
-    OUTPUT_BOARD_ATTRIBUTE_STRING(boardlist_xml, info, last_modified);
-
   if (info->filter.rule != NULL)
     {
       OUTPUT_BOARD_ATTRIBUTE_STRING(boardlist_xml, info, filter.rule);
@@ -1035,7 +1058,7 @@ write_threadlist(gpointer data, gpointer user_data)
   OchushaApplication *application = (OchushaApplication *)user_data;
 
   /* XXX: 顼 */
-  if (!ochusha_bulletin_board_write_threadlist_xml(board, &application->conf))
+  if (!ochusha_bulletin_board_write_threadlist_xml(board, &application->config))
     fprintf(stderr, "Couldn't write threadlist2.xml.\n");
 }
 
@@ -1092,12 +1115,12 @@ build_boardlist(WorkerThread *employee, gpointer args)
       abort();
     }
 
-  buffer = ochusha_utils_2ch_get_bbsmenu(&application->conf, NULL, NULL);
+  buffer = ochusha_utils_2ch_get_bbsmenu(application->broker);
 
   if (buffer == NULL)
     {
       /* XXX: 顼 */
-      fprintf(stderr, "Couldn't get %s\n", application->conf.bbsmenu_url);
+      fprintf(stderr, "Couldn't get %s\n", application->config.bbsmenu_url);
       goto cleanup_before_return;
       return;
     }
@@ -1112,7 +1135,7 @@ build_boardlist(WorkerThread *employee, gpointer args)
       setup_boardlist(application);
       gdk_threads_leave();
 
-      ochusha_bbs_table_write_boardlist_xml(table, &application->conf);
+      ochusha_bbs_table_write_boardlist_xml(table, &application->config);
     }
 
  cleanup_before_return:
