@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.h,v 1.1.1.1 2003/05/10 16:34:25 fuyu Exp $
+ * $Id: boardlist_view.h,v 1.3 2003/05/14 20:58:12 fuyu Exp $
  */
 
 #ifndef _BOARDLIST_VIEW_H_
@@ -79,28 +79,9 @@ struct _BoardlistViewClass
 };
 
 
-typedef struct _BoardCategoryInfo
-{
-  GtkTreeIter iter;
-  gboolean expanded;
-} BoardCategoryInfo;
-
-
-typedef struct _BulletinBoardInfo
-{
-  GtkTreeIter iter;
-} BulletinBoardInfo;
-
-
-typedef BoardCategoryInfo *BoardCategoryInfoNewFunc(OchushaBoardCategory *category);
-typedef BulletinBoardInfo *BulletinBoardInfoNewFunc(OchushaBulletinBoard *board);
-
-
 GType boardlist_view_get_type(void);
 GtkWidget *boardlist_view_new(void);
-void boardlist_view_open(BoardlistView *view, GSList *category_list,
-			 BoardCategoryInfoNewFunc *category_info_factory,
-			 BulletinBoardInfoNewFunc *board_info_factory);
+void boardlist_view_open(BoardlistView *view, GSList *category_list);
 
 void boardlist_view_expand_category(BoardlistView *view,
 				    OchushaBoardCategory *category);
