@@ -23,15 +23,15 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.14 2003/05/19 04:15:24 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.19 2003/06/07 14:35:30 fuyu Exp $
  */
 
 #include "config.h"
 
+#include "ochusha_private.h"
 #include "ochusha.h"
-#include "ochusha_ui.h"
 
-#include "ochusha_thread_2ch.h"
+#include "ochusha_ui.h"
 
 #include "bbs_thread_ui.h"
 #include "bbs_thread_view.h"
@@ -212,9 +212,6 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
 
   info->view_flags = ochusha_utils_get_attribute_int(hash_table, "view_flags");
   info->view_rank = ochusha_utils_get_attribute_int(hash_table, "view_rank");
-
-  info->last_modified = ochusha_utils_get_attribute_string(hash_table,
-							   "last_modified");
   info->view_ignored = ochusha_utils_get_attribute_int(hash_table,
 						       "view_ignored");
 
@@ -248,14 +245,27 @@ threadlist_write_thread_element_cb(OchushaBulletinBoard *board,
       OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info,
 				  view_flags);
       OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info, view_rank);
-      if (info->last_modified != NULL)
-	OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, info, last_modified);
-
       OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info, view_ignored);
     }
 }
 
 
+static void
+bulletin_board_gui_info_free(BulletinBoardGUIInfo *info)
+{
+  if (info->filter.rule != NULL)
+    {
+      G_FREE(info->filter.rule);
+      info->filter.rule = NULL;
+    }
+
+  if (info->properties_dialog != NULL)
+    gtk_widget_destroy(info->properties_dialog);
+
+  G_FREE(info);
+}
+
+
 BulletinBoardGUIInfo *
 ensure_bulletin_board_info(OchushaBulletinBoard *board)
 {
@@ -265,9 +275,10 @@ ensure_bulletin_board_info(OchushaBulletinBoard *board)
       info = g_object_get_qdata(G_OBJECT(board), board_info_id);
       if (info == NULL)
 	{
-	  info = g_new0(BulletinBoardGUIInfo, 1);
+	  info = G_NEW0(BulletinBoardGUIInfo, 1);
 	  g_object_set_qdata_full(G_OBJECT(board), board_info_id,
-				  info, g_free);
+				info,
+				(GDestroyNotify)bulletin_board_gui_info_free);
 	  g_signal_connect(G_OBJECT(board), "threadlist_read_thread_element",
 			   G_CALLBACK(threadlist_read_thread_element_cb),
 			   NULL);
@@ -318,13 +329,13 @@ title_match_func(const gchar *title, ThreadlistSearchQueryData *search_data)
     {
       native_string = convert_string(utf8_to_native, title, -1);
       fprintf(stderr, "title_match_func: title=\"%s\"\n", native_string);
-      g_free(native_string);
+      G_FREE(native_string);
     }
   if (search_data->key != NULL)
     {
       native_string = convert_string(utf8_to_native, search_data->key, -1);
       fprintf(stderr, "title_match_func: key=\"%s\"\n", native_string);
-      g_free(native_string);
+      G_FREE(native_string);
     }
 #endif
   if (search_data->key == NULL)
@@ -335,8 +346,13 @@ title_match_func(const gchar *title, ThreadlistSearchQueryData *search_data)
   if (!search_data->match_case)
     {
       gchar *case_normalized_title = g_utf8_casefold(normalized_title, -1);
-      g_free(normalized_title);
+      G_FREE(normalized_title);
+#if TRACE_MEMORY_USAGE
+      normalized_title = G_STRDUP(case_normalized_title);
+      g_free(case_normalized_title);
+#else
       normalized_title = case_normalized_title;
+#endif
     }
 
 #if ENABLE_REGEXP
@@ -354,7 +370,7 @@ title_match_func(const gchar *title, ThreadlistSearchQueryData *search_data)
   result = (strstr(normalized_title, search_data->key) != NULL);
 #endif
 
-  g_free(normalized_title);
+  G_FREE(normalized_title);
 
   return result;
 }
@@ -392,22 +408,8 @@ open_bulletin_board(OchushaApplication *application,
 
       view = threadlist_view_new();
 
-      job = (WorkerJob *)calloc(1, sizeof(WorkerJob));
-      job_args = (ThreadlistJobArgs *)calloc(1, sizeof(ThreadlistJobArgs));
-      if (job == NULL || job_args == NULL)
-	{
-	  /* Out of memoryǥåɥꥹȤɽ򤢤 */
-	  fprintf(stderr, "Couldn't start show thread list worker.\n");
-	  g_object_unref(G_OBJECT(view));
-
-	  if (job != NULL)
-	    free(job);
-	  if (job_args != NULL)
-	    free(job_args);
-
-	  THREAD_LIST_UNLOCK;
-	  return NULL;
-	}
+      job = G_NEW0(WorkerJob, 1);
+      job_args = G_NEW0(ThreadlistJobArgs, 1);
 
       gtk_widget_show(view);
       paned_notebook
@@ -508,11 +510,10 @@ open_bulletin_board(OchushaApplication *application,
       info->is_busy = TRUE;
       info->subject_txt_buffer
 	= ochusha_bulletin_board_get_threadlist_source(board,
-						&application->conf,
-						info->last_modified,
-						OCHUSHA_CACHE_TRY_REFRESH,
-						&icon_indicator_funcs,
-						tab_label);
+				application->broker,
+				OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESH);
+      if (info->subject_txt_buffer != NULL)
+	setup_for_tab_label_animation(info->subject_txt_buffer, tab_label);
 
       /* Ȥޤǳ */
       g_object_ref(G_OBJECT(tab_label));
@@ -562,15 +563,8 @@ ochusha_open_thread(OchushaApplication *application,
    * Ǥϡ򳫤ΤϤθˤʤΤǡĤΤλ
    * Ԥθ她򳫤ΥåɤȤ
    */
-  job = (WorkerJob *)calloc(1, sizeof(WorkerJob));
-  args = (OpenThreadJobArgs *)calloc(1, sizeof(OpenThreadJobArgs));
-  if (job == NULL || args == NULL)
-    {
-      if (job != NULL)
-	free(job);
-      if (args != NULL)
-	free(args);
-    }
+  job = G_NEW0(WorkerJob, 1);
+  args = G_NEW0(OpenThreadJobArgs, 1);
 
   job->canceled = FALSE;
   job->job = open_thread_dat;
@@ -579,9 +573,10 @@ ochusha_open_thread(OchushaApplication *application,
   args->application = application;
   args->board = board;
   args->thread = thread;
-  args->dat_filename = g_strdup(dat_filename);
+  args->dat_filename = G_STRDUP(dat_filename);
   args->res_num = res_num;
   args->in_tab = in_tab;
+
   commit_job(job);
 }
 
@@ -621,8 +616,8 @@ open_thread_dat(WorkerThread *employee, gpointer job_args)
     }
   THREAD_LIST_UNLOCK;
 
-  g_free(args->dat_filename);
-  g_free(args);
+  G_FREE(args->dat_filename);
+  G_FREE(args);
 }
 
 
@@ -665,11 +660,8 @@ refresh_threadlist_view(OchushaApplication *application,
 
       info->subject_txt_buffer
 	= ochusha_bulletin_board_get_threadlist_source(board,
-						&application->conf,
-						info->last_modified,
-						OCHUSHA_CACHE_TRY_REFRESH,
-						&icon_indicator_funcs,
-						tab_label);
+				application->broker,
+				OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESH);
       if (info->subject_txt_buffer == NULL)
 	{
 	  info->subject_txt_buffer = old_buffer;
@@ -677,19 +669,10 @@ refresh_threadlist_view(OchushaApplication *application,
 	  return;
 	}
 
-      job = (WorkerJob *)calloc(1, sizeof(WorkerJob));
-      job_args = (ThreadlistJobArgs *)calloc(1, sizeof(ThreadlistJobArgs));
-      if (job == NULL || job_args == NULL)
-	{
-	  if (job != NULL)
-	    free(job);
-	  if (job_args != NULL)
-	    free(job_args);
-	  fprintf(stderr, "Couldn't start update thread list worker.\n");
-	  info->subject_txt_buffer = old_buffer;
-	  THREAD_LIST_UNLOCK;
-	  return;
-	}
+      setup_for_tab_label_animation(info->subject_txt_buffer, tab_label);
+
+      job = G_NEW0(WorkerJob, 1);
+      job_args = G_NEW0(ThreadlistJobArgs, 1);
 
       job_args->application = application;
       job_args->board = board;
@@ -751,18 +734,8 @@ redraw_threadlist_view(OchushaApplication *application,
 	  return;
 	}
 
-      job = (WorkerJob *)calloc(1, sizeof(WorkerJob));
-      job_args = (ThreadlistJobArgs *)calloc(1, sizeof(ThreadlistJobArgs));
-      if (job == NULL || job_args == NULL)
-	{
-	  if (job != NULL)
-	    free(job);
-	  if (job_args != NULL)
-	    free(job_args);
-	  fprintf(stderr, "Couldn't start update thread list worker.\n");
-	  THREAD_LIST_UNLOCK;
-	  return;
-	}
+      job = G_NEW0(WorkerJob, 1);
+      job_args = G_NEW0(ThreadlistJobArgs, 1);
 
       job_args->application = application;
       job_args->board = board;
@@ -788,6 +761,7 @@ redraw_threadlist_view(OchushaApplication *application,
 typedef struct _EachThreadArgs
 {
   OchushaApplication *application;
+  BulletinBoardGUIInfo *info;
   ThreadlistView *view;
   int number_of_threads;
 
@@ -806,38 +780,56 @@ each_thread_cb(OchushaBBSThread *thread, gpointer user_data)
   ThreadlistView *view = args->view;
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   gboolean filter_result;
+  gboolean update_condition
+    = (!args->application->config.offline && args->increment_ignored);
 
 #if DEBUG_GUI_MOST
   gchar *title = convert_string(utf8_to_native, thread->title, -1);
   fprintf(stderr, "each_thread_cb: %s <%d>\n",
 	  title, thread->number_of_responses_on_server);
-  free(title);
+  G_FREE(title);
 #endif
 
   args->number_of_threads++;
 
+  if (update_condition)
+    info->view_flags &= ~BBS_THREAD_NEW;
+
+  if (info->view_rank == 0)
+    info->view_flags |= BBS_THREAD_NEW;
+
+  if ((info->view_flags & BBS_THREAD_NEW) != 0)
+    info->view_rank = 0;	/* ĥ뤿 */
+
   filter_result = threadlist_filter_check_thread(args->filter,
 						 args->application, thread);
-
   if (filter_result || !args->enable_filter)
     {
-      if (info->view_rank == 0)
-	info->view_flags |= BBS_THREAD_NEW;
-      else if (args->increment_ignored && !args->application->conf.offline)
-	info->view_flags &= ~BBS_THREAD_NEW;
-
       gdk_threads_enter();
+
+#if ENABLE_RENDERING_ON_IDLE
+      while (!try_get_rendering_right(args->application,
+				      args->info->subject_txt_buffer,
+				      thread->board))
+	{
+	  gdk_threads_leave();
+	  ochusha_async_buffer_wait(args->info->subject_txt_buffer,
+				    "bulletin_board_ui.c: each_thread_cb()");
+	  gdk_threads_enter();
+	}
+#endif
+
       threadlist_view_append_thread(view, thread,
 				    info->view_rank, args->number_of_threads);
       update_threadlist_entry_style(args->application, view, thread);
+
       gdk_threads_leave();
       args->last_thread = thread;
-      if (!args->application->conf.offline && args->increment_ignored)
+      if (update_condition)
 	info->view_ignored++;
+      info->view_rank = args->number_of_threads;
     }
 
-  info->view_rank = args->number_of_threads;
-
   return TRUE;
 }
 
@@ -854,6 +846,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
   EachThreadArgs cb_args =
     {
       application,
+      info,
       view,
       0,			/* number_of_threads */
       NULL,			/* OchushaBBSThread *last_thread */
@@ -865,12 +858,12 @@ show_threadlist(WorkerThread *employee, gpointer args)
   {
     gchar *board_name = convert_string(utf8_to_native, board->name, -1);
     fprintf(stderr, "refresh_current_threadlist: update threadlist for board: %s.\n", board_name);
-    free(board_name);
+    G_FREE(board_name);
   }
 #endif
 
   if (board->thread_list == NULL)
-    ochusha_bulletin_board_read_threadlist_xml(board, &application->conf);
+    ochusha_bulletin_board_read_threadlist_xml(board, &application->config);
 
   THREAD_LIST_LOCK
     {
@@ -883,7 +876,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
 #if DEBUG_GUI_MOST
       gchar *name = convert_string(utf8_to_native, board->name, -1);
       fprintf(stderr, "Couldn't get thread list (subject.txt) for %s\n", name);
-      free(name);
+      G_FREE(name);
 #endif
       goto finish_job;
     }
@@ -905,7 +898,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
 #if DEBUG_GUI_MOST
       gchar *name = convert_string(utf8_to_native, board->name, -1);
       fprintf(stderr, "Broken thread list (subject.txt) for %s\n", name);
-      free(name);
+      G_FREE(name);
 #endif
       gdk_threads_enter();
       threadlist_view_close(view, NULL);
@@ -923,31 +916,13 @@ show_threadlist(WorkerThread *employee, gpointer args)
 
  finish_job:
 #if DEBUG_ASYNC_BUFFER_MOST
-  fprintf(stderr, "show_threadlist: unref OchushaAsyncBuffer(0x%x)\n",
-	  (int)buffer);
+  fprintf(stderr, "show_threadlist: unref OchushaAsyncBuffer(%p)\n",
+	  buffer);
   fprintf(stderr, "* before unref: buffer->ref_count=%d\n",
 	  G_OBJECT(buffer)->ref_count);
 #endif
   THREAD_LIST_LOCK
     {
-      OchushaNetworkStatus *status = (OchushaNetworkStatus *)buffer->user_data;
-      if (status != NULL)
-	{
-	  if (status->last_modified != NULL)
-	    {
-	      if (info->last_modified != NULL)
-		free(info->last_modified);
-	      info->last_modified = g_strdup(status->last_modified);
-	    }
-	  else
-	    if (status->state == OCHUSHA_ACCESS_FAILED)
-	      {
-		if (info->last_modified != NULL)
-		  free(info->last_modified);
-		info->last_modified = NULL;
-	      }
-	}
-
       info->is_busy = FALSE;
       THREAD_LIST_COND_BROADCAST;
     }
@@ -962,7 +937,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
   if (buffer != NULL)
     g_object_unref(G_OBJECT(buffer));
 
-  free(args);
+  G_FREE(args);
 }
 
 
@@ -1090,8 +1065,8 @@ thread_mouse_over_cb(ThreadlistView *view, OchushaBBSThread *thread,
   gdk_window_get_pointer(gdk_screen_get_root_window(gtk_widget_get_screen(GTK_WIDGET(view))),
 			 &title_popup_x_pos, &title_popup_y_pos, NULL);
 #if DEBUG_GUI
-  fprintf(stderr, "thread_mouse_over_cb: x=%d, y=%d, thread=0x%x\n",
-	  title_popup_x_pos, title_popup_y_pos, (int)thread);
+  fprintf(stderr, "thread_mouse_over_cb: x=%d, y=%d, thread=%p\n",
+	  title_popup_x_pos, title_popup_y_pos, thread);
 #endif
 
   THREAD_LIST_LOCK
@@ -1112,7 +1087,7 @@ thread_mouse_out_cb(ThreadlistView *view, OchushaBBSThread *thread,
 		    OchushaApplication *application)
 {
 #if DEBUG_GUI
-  fprintf(stderr, "thread_mouse_out_cb: thread=0x%x\n", (int)thread);
+  fprintf(stderr, "thread_mouse_out_cb: thread=%p\n", thread);
 #endif
   THREAD_LIST_LOCK
     {
@@ -1132,8 +1107,8 @@ thread_title_popup_timeout(gpointer data)
   gboolean result = TRUE;
 
 #if DEBUG_GUI
-  fprintf(stderr, "thread_title_popup_timeout: x=%d, y=%d, popup_thread=0x%x, data=0x%x\n",
-	  title_popup_x_pos, title_popup_y_pos, (int)popup_thread, (int)data);
+  fprintf(stderr, "thread_title_popup_timeout: x=%d, y=%d, popup_thread=%p, data=%p\n",
+	  title_popup_x_pos, title_popup_y_pos, popup_thread, data);
 #endif
 
   gdk_threads_enter();
@@ -1209,8 +1184,8 @@ free_dat_buffer(OchushaBBSThread *thread, gpointer unused)
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
 
 #if DEBUG_ASYNC_BUFFER_MOST
-  fprintf(stderr, "free_dat_buffer: unref OchushaAsyncBuffer(0x%x)\n",
-	  (int)info->dat_buffer);
+  fprintf(stderr, "free_dat_buffer: unref OchushaAsyncBuffer(%p)\n",
+	  info->dat_buffer);
   fprintf(stderr, "* before unref: dat_buffer->ref_count=%d\n",
 	  G_OBJECT(info->dat_buffer)->ref_count);
 #endif
@@ -1241,8 +1216,8 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook, OchushaBBSThread *threa
    */
 
 #if DEBUG_GUI
-  fprintf(stderr, "bulletin_board_ui.c: being_closed thread=0x%x, item_view=0x%x\n",
-	  (int)thread, (int)item_view);
+  fprintf(stderr, "bulletin_board_ui.c: being_closed thread=%p, item_view=%p\n",
+	  thread, item_view);
 #endif
 
 #ifndef SUSPEND_ALL_FOR_DESTRUCTION	/* ¸ */
@@ -1311,8 +1286,7 @@ item_view_closed_cb(PanedNotebook *paned_notebook, OchushaBBSThread *thread,
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   /* ΥåɽĤ줿 */
 #if DEBUG_GUI_MOST
-  fprintf(stderr, "closed: thread=0x%x, item_view=0x%x\n",
-	  (int)thread, (int)item_view);
+  fprintf(stderr, "closed: thread=%p, item_view=%p\n", thread, item_view);
 #endif
 
 #ifndef SUSPEND_ALL_FOR_DESTRUCTION
@@ -1449,10 +1423,9 @@ write_response(OchushaApplication *application)
     /*
      * TODO: ǽ񤭹ߤޤ礦
      */
-    char url[PATH_MAX];
-    snprintf(url, PATH_MAX, "%sl10",
-	     ochusha_bbs_thread_get_thread_url(thread));
-    ochusha_open_url(application, url, FALSE, TRUE);
+    ochusha_open_url(application,
+		     ochusha_bbs_thread_get_url_to_post_response(thread),
+		     FALSE, TRUE);
   }
 }
 
