@@ -23,17 +23,18 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: main.c,v 1.1.1.1 2003/05/10 16:34:25 fuyu Exp $
+ * $Id: main.c,v 1.4 2003/06/07 14:35:30 fuyu Exp $
  */
 
 #include "config.h"
 
+#include "ochusha_private.h"
 #include "ochusha.h"
-#include "ochusha_ui.h"
-
 #include "htmlutils.h"
-
 #include "worker.h"
+#include "utils.h"
+
+#include "ochusha_ui.h"
 
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -50,10 +51,14 @@ int
 main(int argc, char *argv[])
 {
   gchar *resource_file;
-  OchushaApplication *application
-    = (OchushaApplication *)calloc(1, sizeof(OchushaApplication));
+  OchushaApplication *application;
 
-  ochusha_config_prepare_home(&application->conf);
+#if TRACE_MEMORY_USAGE
+  init_memory_trace();
+#endif
+
+  application = G_NEW0(OchushaApplication, 1);
+  ochusha_config_prepare_home(&application->config);
 
   g_thread_init(NULL);
   gdk_threads_init();
@@ -68,20 +73,53 @@ main(int argc, char *argv[])
   initialize_common_converters();
   initialize_worker(INITIAL_NUM_THREADS, MAXIMUM_NUM_THREADS);
 
-  resource_file = ochusha_config_find_file(&application->conf, OCHUSHA_GTKRC);
+  resource_file = ochusha_config_find_file(&application->config,
+					   OCHUSHA_GTKRC);
   if (resource_file != NULL)
     {
       gtk_rc_parse(resource_file);
-      g_free(resource_file);
+      G_FREE(resource_file);
     }
 
   ochusha_init_application(application);
 
   gdk_threads_enter();
   gtk_main();
+
+#if TRACE_MEMORY_USAGE
+  if (application->top_level != NULL)
+    gtk_widget_destroy(GTK_WIDGET(application->top_level));
+#endif
+
   gdk_threads_leave();
 
   terminate_workers();
 
+#if TRACE_MEMORY_USAGE
+  g_object_unref(G_OBJECT(application->table));
+  application->table = NULL;
+  g_object_unref(G_OBJECT(application->broker));
+  application->broker = NULL;
+
+  G_FREE(application->user_default_filter.rule);
+  application->user_default_filter.rule = NULL;
+  G_FREE(application->web_browser_template);
+  application->web_browser_template = NULL;
+  G_FREE(application->config.home);
+  application->config.home = NULL;
+  G_FREE(application->config.bbsmenu_url);
+  application->config.bbsmenu_url = NULL;
+  G_FREE(application->config.proxy_url);
+  application->config.proxy_url = NULL;
+  G_FREE(application->config.proxy_user);
+  application->config.proxy_user = NULL;
+  G_FREE(application->config.proxy_password);
+  application->config.proxy_password = NULL;
+
+  G_FREE(application);
+
+  dump_pointers();
+#endif
+
   return 0;
 }
