@@ -9,19 +9,22 @@ G_BEGIN_DECLS
 /* VOID:BOOLEAN (marshal.list:1) */
 #define ochusha_marshal_VOID__BOOLEAN	g_cclosure_marshal_VOID__BOOLEAN
 
-/* VOID:INT (marshal.list:2) */
+/* VOID:BOXED (marshal.list:2) */
+#define ochusha_marshal_VOID__BOXED	g_cclosure_marshal_VOID__BOXED
+
+/* VOID:INT (marshal.list:3) */
 #define ochusha_marshal_VOID__INT	g_cclosure_marshal_VOID__INT
 
-/* VOID:OBJECT (marshal.list:3) */
+/* VOID:OBJECT (marshal.list:4) */
 #define ochusha_marshal_VOID__OBJECT	g_cclosure_marshal_VOID__OBJECT
 
-/* VOID:POINTER (marshal.list:4) */
+/* VOID:POINTER (marshal.list:5) */
 #define ochusha_marshal_VOID__POINTER	g_cclosure_marshal_VOID__POINTER
 
-/* VOID:VOID (marshal.list:5) */
+/* VOID:VOID (marshal.list:6) */
 #define ochusha_marshal_VOID__VOID	g_cclosure_marshal_VOID__VOID
 
-/* VOID:ENUM,INT (marshal.list:6) */
+/* VOID:ENUM,INT (marshal.list:7) */
 extern void ochusha_marshal_VOID__ENUM_INT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -29,7 +32,7 @@ extern void ochusha_marshal_VOID__ENUM_INT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:OBJECT,OBJECT (marshal.list:7) */
+/* VOID:OBJECT,OBJECT (marshal.list:8) */
 extern void ochusha_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -37,7 +40,7 @@ extern void ochusha_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:POINTER,BOOLEAN (marshal.list:8) */
+/* VOID:POINTER,BOOLEAN (marshal.list:9) */
 extern void ochusha_marshal_VOID__POINTER_BOOLEAN (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -45,7 +48,7 @@ extern void ochusha_marshal_VOID__POINTER_BOOLEAN (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER (marshal.list:9) */
+/* VOID:POINTER,POINTER (marshal.list:10) */
 extern void ochusha_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -53,7 +56,7 @@ extern void ochusha_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:BOXED,POINTER,STRING (marshal.list:10) */
+/* VOID:BOXED,POINTER,STRING (marshal.list:11) */
 extern void ochusha_marshal_VOID__BOXED_POINTER_STRING (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -61,7 +64,7 @@ extern void ochusha_marshal_VOID__BOXED_POINTER_STRING (GClosure     *closure,
                                                         gpointer      invocation_hint,
                                                         gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER,POINTER,POINTER (marshal.list:11) */
+/* VOID:POINTER,POINTER,POINTER,POINTER (marshal.list:12) */
 extern void ochusha_marshal_VOID__POINTER_POINTER_POINTER_POINTER (GClosure     *closure,
                                                                    GValue       *return_value,
                                                                    guint         n_param_values,
@@ -69,7 +72,7 @@ extern void ochusha_marshal_VOID__POINTER_POINTER_POINTER_POINTER (GClosure
                                                                    gpointer      invocation_hint,
                                                                    gpointer      marshal_data);
 
-/* BOOLEAN:ENUM,INT (marshal.list:12) */
+/* BOOLEAN:ENUM,INT (marshal.list:13) */
 extern void ochusha_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -77,7 +80,7 @@ extern void ochusha_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* BOOLEAN:POINTER (marshal.list:13) */
+/* BOOLEAN:POINTER (marshal.list:14) */
 extern void ochusha_marshal_BOOLEAN__POINTER (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -85,7 +88,7 @@ extern void ochusha_marshal_BOOLEAN__POINTER (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* BOOLEAN:BOXED,POINTER,STRING (marshal.list:14) */
+/* BOOLEAN:BOXED,POINTER,STRING (marshal.list:15) */
 extern void ochusha_marshal_BOOLEAN__BOXED_POINTER_STRING (GClosure     *closure,
                                                            GValue       *return_value,
                                                            guint         n_param_values,
@@ -93,7 +96,7 @@ extern void ochusha_marshal_BOOLEAN__BOXED_POINTER_STRING (GClosure     *closure
                                                            gpointer      invocation_hint,
                                                            gpointer      marshal_data);
 
-/* BOOLEAN:STRING,INT,BOOLEAN,BOOLEAN,BOOLEAN (marshal.list:15) */
+/* BOOLEAN:STRING,INT,BOOLEAN,BOOLEAN,BOOLEAN (marshal.list:16) */
 extern void ochusha_marshal_BOOLEAN__STRING_INT_BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                                          GValue       *return_value,
                                                                          guint         n_param_values,
