@@ -23,17 +23,19 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.c,v 1.8 2003/05/15 19:35:01 fuyu Exp $
+ * $Id: ochusha_ui.c,v 1.13 2003/06/07 14:35:31 fuyu Exp $
  */
 
 #include "config.h"
 
+#include "ochusha_private.h"
 #include "ochusha.h"
-#define OCHUSHA_UI_PRIVATE
-#include "ochusha_ui.h"
-
+#include "ochusha_bbs_table.h"
+#include "ochusha_network_broker.h"
 #include "ochusha_utils_2ch.h"
+#include "worker.h"
 
+#include "ochusha_ui.h"
 #include "bbs_thread_ui.h"
 #include "boardlist_ui.h"
 #include "bulletin_board_ui.h"
@@ -43,8 +45,6 @@
 
 #include "ugly_gtk2utils.h"
 
-#include "worker.h"
-
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
@@ -85,11 +85,25 @@ static void write_optional_prefs(FILE *file, gpointer user_data);
 static void configure_application(OchushaApplication *application);
 static void read_optional_prefs(GHashTable *pref_attrs, gpointer user_data);
 
-static void access_started_cb(gpointer user_data);
-static void access_progressed_cb(gpointer user_data);
-static void access_completed_cb(gpointer user_data);
-static void access_terminated_cb(gpointer user_data);
-static void access_failed_cb(gpointer user_data);
+
+static gboolean access_started_cb(OchushaNetworkBroker *broker,
+				  OchushaAsyncBuffer *buffer,
+				  OchushaApplication *application);
+static gboolean access_progressed_cb(OchushaNetworkBroker *broker,
+				     OchushaAsyncBuffer *buffer,
+				     int bytes_read, int bytes_total,
+				     OchushaApplication *application);
+static gboolean access_completed_cb(OchushaNetworkBroker *broker,
+				    OchushaAsyncBuffer *buffer,
+				    OchushaApplication *application);
+static gboolean access_terminated_cb(OchushaNetworkBroker *broker,
+				     OchushaAsyncBuffer *buffer,
+				     OchushaApplication *application);
+static gboolean access_failed_cb(OchushaNetworkBroker *broker,
+				 OchushaAsyncBuffer *buffer,
+				 OchushaNetworkBrokerFailureReason reason_code,
+				 const gchar *reason,
+				 OchushaApplication *application);
 
 static void open_about_dialog(OchushaApplication *application);
 static void about_dialog_response_cb(GtkWidget *dialog, gint response_id,
@@ -103,21 +117,11 @@ static void open_url_dialog_response_cb(GtkWidget *dialog, gint response_id,
 static void popup_open_url_dialog(OchushaApplication *application);
 
 
-OchushaNetworkCallbacks icon_indicator_funcs =
-{
-  access_started_cb,
-  access_progressed_cb,
-  access_completed_cb,
-  access_terminated_cb,
-  access_failed_cb
-};
-
-
 static GtkItemFactory *main_menu_item_factory;
 static GdkPixbuf *tab_label_icons[7];
+static GQuark tab_label_id;
 static GQuark tab_label_icon_counter_id;
 
-
 static pthread_mutex_t main_ui_lock;
 
 #define MAIN_UI_LOCK					\
@@ -268,6 +272,10 @@ ochusha_init_application(OchushaApplication *application)
   application->top_level = GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));
   g_signal_connect(G_OBJECT(application->top_level), "delete_event",
 		   G_CALLBACK(delete_event_cb), application);
+#if TRACE_MEMORY_USAGE
+  g_signal_connect(G_OBJECT(application->top_level), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed), &application->top_level);
+#endif
 
   gtk_window_set_title(application->top_level, _("ochusha"));
 
@@ -278,6 +286,11 @@ ochusha_init_application(OchushaApplication *application)
   application->main_window = GTK_BOX(gtk_vbox_new(FALSE, 0));
   gtk_container_add(GTK_CONTAINER(application->top_level),
 		    GTK_WIDGET(application->main_window));
+#if TRACE_MEMORY_USAGE
+  g_signal_connect(G_OBJECT(application->main_window), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed),
+		   &application->main_window);
+#endif
 
   gtk_window_set_geometry_hints(application->top_level,
 				GTK_WIDGET(application->main_window),
@@ -299,10 +312,17 @@ ochusha_init_application(OchushaApplication *application)
   /* ochusha_set_main_menu_sensitive(MAIN_MENU_OPEN_URL, FALSE); */
   ochusha_set_main_menu_sensitive(MAIN_MENU_REFRESH_THREAD_ALL, FALSE);
   /* ochusha_set_main_menu_sensitive(MAIN_MENU_ABOUT, FALSE); */
-
+#if TRACE_MEMORY_USAGE
+  g_signal_connect(G_OBJECT(application->menu_bar), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed), &application->menu_bar);
+#endif
 
   gtk_box_pack_start(application->main_window,
 		     application->menu_bar, FALSE, TRUE, 0);
+#if TRACE_MEMORY_USAGE
+  g_signal_connect(G_OBJECT(application->menu_bar), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed), &application->menu_bar);
+#endif
 
 #if ENABLE_MAIN_TOOLBAR
   /* toolbarŹ */
@@ -311,6 +331,10 @@ ochusha_init_application(OchushaApplication *application)
   gtk_box_pack_start(application->main_window,
 		     GTK_WIDGET(application->toolbar),
 		     FALSE, TRUE, 0);
+#if TRACE_MEMORY_USAGE
+  g_signal_connect(G_OBJECT(application->toolbar), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed), &application->toolbar);
+#endif
 
   gtk_toolbar_insert_stock(application->toolbar, GTK_STOCK_GO_FORWARD,
 			   _("Go next page"),
@@ -341,7 +365,11 @@ ochusha_init_application(OchushaApplication *application)
 			    GTK_WIDGET(application->url_box),
 			    _("Input URL to show"),
 			    "input_url_to_show");
+#if TRACE_MEMORY_USAGE
+  g_signal_connect(G_OBJECT(application->url_box), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed), &application->url_box);
 #endif
+#endif	/* ENABLE_MAIN_TOOLBAR */
   /* ν */
   initialize_open_url_dialog(application);
 
@@ -357,6 +385,11 @@ ochusha_init_application(OchushaApplication *application)
 
   gtk_box_pack_start(application->main_window,
 		     application->contents_window, TRUE, TRUE, 0);
+#if TRACE_MEMORY_USAGE
+  g_signal_connect(G_OBJECT(application->contents_window), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed),
+		   &application->contents_window);
+#endif
 
   application->statusbar = GTK_STATUSBAR(gtk_statusbar_new());
 
@@ -365,6 +398,11 @@ ochusha_init_application(OchushaApplication *application)
 
   gtk_box_pack_end(application->main_window,
 		   GTK_WIDGET(application->statusbar), FALSE, TRUE, 0);
+#if TRACE_MEMORY_USAGE
+  g_signal_connect(G_OBJECT(application->statusbar), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed),
+		   &application->statusbar);
+#endif
 
   application->clipboard = NULL;
 
@@ -412,6 +450,10 @@ ochusha_open_url(OchushaApplication *application, const gchar *url,
   unsigned int to;
   char dat_filename[PATH_MAX];
 
+  /*
+   * XXX: Ǥurl2chURLɤĴ٤Ƥۤˤ餷Ƥ뤬
+   *      ʥ2ch¸ʬϤ٤
+   */
   if (use_web_browser
       || !ochusha_utils_2ch_parse_url(application->table, url,
 				      &board, &thread_num, &thread,
@@ -472,13 +514,34 @@ get_current_url(OchushaApplication *application)
 #endif
 
 
-#if DEBUG_GUI_MOST
 static void
-free_quark_data(gpointer data)
+invalidate_tab_label_animation(OchushaAsyncBuffer *buffer)
 {
-  fprintf(stderr, "freeing data(0x%x)\n", (int)data);
-  g_free(data);
+  g_object_set_qdata(G_OBJECT(buffer), tab_label_id, NULL);
 }
+
+
+void
+setup_for_tab_label_animation(OchushaAsyncBuffer *buffer, IconLabel *tab_label)
+{
+  g_return_if_fail(OCHUSHA_IS_ASYNC_BUFFER(buffer)
+		   && IS_ICON_LABEL(tab_label));
+  g_signal_connect(G_OBJECT(tab_label), "destroy",
+		   G_CALLBACK(invalidate_tab_label_animation),
+		   buffer);
+  g_object_set_qdata(G_OBJECT(buffer), tab_label_id, tab_label);
+}
+
+
+#if TRACE_MEMORY_USAGE
+static void
+trace_free(gpointer pointer)
+{
+  G_FREE(pointer);
+}
+#define TRACE_FREE	trace_free
+#else
+#define TRACE_FREE	g_free
 #endif
 
 
@@ -487,13 +550,8 @@ get_tab_label(const gchar *label_text)
 {
   GtkWidget *label = icon_label_new(label_text,
 				tab_label_icons[TAB_LABEL_ICON_PREPARATION]);
-#if DEBUG_GUI_MOST
   g_object_set_qdata_full(G_OBJECT(label), tab_label_icon_counter_id,
-			  g_new(int, 1), (GDestroyNotify)free_quark_data);
-#else
-  g_object_set_qdata_full(G_OBJECT(label), tab_label_icon_counter_id,
-			  g_new(int, 1), (GDestroyNotify)g_free);
-#endif
+			  G_NEW(int, 1), (GDestroyNotify)TRACE_FREE);
   return label;
 }
 
@@ -515,6 +573,7 @@ setup_tab_label_icons(void)
     = gdk_pixbuf_new_from_xpm_data((const char **)black_xpm);
   tab_label_icons[TAB_LABEL_ICON_ERROR]
     = gdk_pixbuf_new_from_xpm_data((const char **)red_xpm);
+  tab_label_id = g_quark_from_static_string("OchushaUI::icon-label");
   tab_label_icon_counter_id
     = g_quark_from_static_string("OchushaUI::icon-counter");
 }
@@ -579,7 +638,7 @@ on_quit(gpointer data, guint action, GtkWidget *widget)
 
   finalize_boardlist(application);
 
-  if (!ochusha_write_config_xml(&application->conf,
+  if (!ochusha_write_config_xml(&application->config,
 				write_optional_prefs, application))
     fprintf(stderr, "Couldn't write %s.\n", OCHUSHA_CONFIG_XML);
 
@@ -628,23 +687,23 @@ main_menu_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 
-#define OUTPUT_CONFIG_ATTRIBUTE_INT(file, conf, attribute)	\
+#define OUTPUT_CONFIG_ATTRIBUTE_INT(file, config, attribute)	\
   fprintf(file,							\
 	  "    <attribute name=\"" #attribute	"\">\n"		\
 	  "      <int val=\"%d\"/>\n"				\
-	  "    </attribute>\n", (conf)->attribute)
+	  "    </attribute>\n", (config)->attribute)
 
-#define OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, conf, attribute)	\
-  fprintf(file,							\
-	  "    <attribute name=\"" #attribute	"\">\n"		\
-	  "      <boolean val=\"%s\"/>\n"			\
-	  "    </attribute>\n", (conf)->attribute ? "true" : "false" )
+#define OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, config, attribute)	\
+  fprintf(file,								\
+	  "    <attribute name=\"" #attribute	"\">\n"			\
+	  "      <boolean val=\"%s\"/>\n"				\
+	  "    </attribute>\n", (config)->attribute ? "true" : "false" )
 
-#define OUTPUT_CONFIG_ATTRIBUTE_STRING(file, conf, attribute)	\
+#define OUTPUT_CONFIG_ATTRIBUTE_STRING(file, config, attribute)	\
   fprintf(file,							\
 	  "    <attribute name=\"" #attribute	"\">\n"		\
 	  "      <string>%s</string>\n"				\
-	  "    </attribute>\n", (conf)->attribute)
+	  "    </attribute>\n", (config)->attribute)
 
 
 static void
@@ -657,6 +716,13 @@ write_optional_prefs(FILE *file, gpointer user_data)
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, boardlist_width);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, threadlist_height);
 
+#if ENABLE_RENDERING_ON_IDLE
+  OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, rendering_unit);
+  OUTPUT_CONFIG_ATTRIBUTE_INT(file, application,
+			      weight_of_threadlist_rendering);
+  OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, weight_of_response_rendering);
+#endif
+
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, boardlist_pane_style);
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application, board_tab_shrinkable);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, board_tab_minimum_size);
@@ -676,6 +742,7 @@ write_optional_prefs(FILE *file, gpointer user_data)
 
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application, enable_popup_response);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, popup_response_delay);
+  OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, popup_close_delay);
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
 				  enable_smart_popup_placement);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application,
@@ -694,27 +761,84 @@ write_optional_prefs(FILE *file, gpointer user_data)
 static void
 configure_application(OchushaApplication *application)
 {
-  ochusha_read_config_xml(&application->conf, read_optional_prefs,
+  ochusha_read_config_xml(&application->config, read_optional_prefs,
 			  application);
+  application->broker = ochusha_network_broker_new(&application->config);
+  g_signal_connect(G_OBJECT(application->broker), "access_started",
+		   G_CALLBACK(access_started_cb), application);
+  g_signal_connect(G_OBJECT(application->broker), "access_progressed",
+		   G_CALLBACK(access_progressed_cb), application);
+  g_signal_connect(G_OBJECT(application->broker), "access_completed",
+		   G_CALLBACK(access_completed_cb), application);
+  g_signal_connect(G_OBJECT(application->broker), "access_terminated",
+		   G_CALLBACK(access_terminated_cb), application);
+  g_signal_connect(G_OBJECT(application->broker), "access_failed",
+		   G_CALLBACK(access_failed_cb), application);
   /* Sets unspecified attributes */
   if (application->width == 0)
-    {
+    {	/* ƵưȻפ */
+      /* ɥν */
       application->width = OCHUSHA_DEFAULT_WIDTH;
       application->height = OCHUSHA_DEFAULT_HEIGHT;
       application->boardlist_width = OCHUSHA_DEFAULT_BOARDLIST_WIDTH;
       application->threadlist_height = OCHUSHA_DEFAULT_THREADLIST_HEIGHT;
 
-      application->boardlist_pane_style = HPANED_STYLE;
-      application->threadlist_pane_style = VPANED_STYLE;
+#if ENABLE_RENDERING_ON_IDLE
+      application->rendering_unit
+	= OCHUSHA_DEFAULT_RENDERING_UNIT;
+      application->weight_of_threadlist_rendering
+	= OCHUSHA_DEFAULT_WEIGHT_OF_THREADLIST_RENDERING;
+      application->weight_of_response_rendering
+	= OCHUSHA_DEFAULT_WEIGHT_OF_RESPONSE_RENDERING;
+#endif
 
+      /* İϢ */
+      application->boardlist_pane_style = HPANED_STYLE;
+      application->board_tab_shrinkable = OCHUSHA_DEFAULT_BOARD_TAB_SHRINKABLE;
+      application->board_tab_minimum_size
+	= OCHUSHA_DEFAULT_BOARD_TAB_MINIMUM_SIZE;
+      application->board_tab_enable_tooltips = FALSE;
+
+
+      /* Ϣ */
+      application->threadlist_pane_style = VPANED_STYLE;
+      application->thread_tab_shrinkable
+	= OCHUSHA_DEFAULT_THREAD_TAB_SHRINKABLE;
+      application->thread_tab_minimum_size
+	= OCHUSHA_DEFAULT_THREAD_TAB_MINIMUM_SIZE;
+      application->thread_tab_enable_tooltips = TRUE;
+      application->show_threadlist_entry_when_thread_selected = FALSE;
+
+
+      /* 쥿Υݥåץå״Ϣ */
+      application->enable_popup_title = FALSE;
+      application->popup_title_delay = OCHUSHA_DEFAULT_POPUP_DELAY;
+
+
+      /* 쥹Υݥåץå״Ϣ */
+      application->enable_popup_response = TRUE;
+      application->popup_response_delay = OCHUSHA_DEFAULT_POPUP_DELAY;
+      application->popup_close_delay = OCHUSHA_DEFAULT_POPUP_CLOSE_DELAY;
+      application->enable_smart_popup_placement = FALSE;
       application->maximum_number_of_popup_responses
 	= OCHUSHA_DEFAULT_MAXIMUM_NUMBER_OF_POPUP_RESPONSES;
 
-      application->popup_response_delay = OCHUSHA_DEFAULT_POPUP_DELAY;
       application->show_threadlist_entry_when_thread_selected
 	= OCHUSHA_DEFAULT_SHOW_THREADLIST_ENTRY_WHEN_THREAD_SELECTED;
     }
 
+#if ENABLE_RENDERING_ON_IDLE
+  if (application->rendering_unit < 0)
+    application->rendering_unit = 0;
+
+  if (application->weight_of_threadlist_rendering < 0)
+    application->weight_of_threadlist_rendering = 0;
+
+  if (application->weight_of_response_rendering < 0)
+    application->weight_of_response_rendering = 0;
+#endif
+
+
   if (application->board_tab_minimum_size == 0)
     application->board_tab_minimum_size
       = OCHUSHA_DEFAULT_BOARD_TAB_MINIMUM_SIZE;
@@ -746,6 +870,31 @@ read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
   application->threadlist_height
     = ochusha_utils_get_attribute_int(pref_attrs, "threadlist_height");
 
+#if ENABLE_RENDERING_ON_IDLE
+  if (g_hash_table_lookup(pref_attrs, "rendering_unit") != NULL)
+    application->rendering_unit
+      = ochusha_utils_get_attribute_int(pref_attrs, "rendering_unit");
+  else
+    application->rendering_unit = OCHUSHA_DEFAULT_RENDERING_UNIT;
+
+  if (g_hash_table_lookup(pref_attrs,
+			  "weight_of_threadlist_rendering") != NULL)
+    application->weight_of_threadlist_rendering
+      = ochusha_utils_get_attribute_int(pref_attrs,
+					"weight_of_threadlist_rendering");
+  else
+    application->weight_of_threadlist_rendering
+      = OCHUSHA_DEFAULT_WEIGHT_OF_THREADLIST_RENDERING;
+  
+  if (g_hash_table_lookup(pref_attrs, "weight_of_response_rendering") != NULL)
+    application->weight_of_response_rendering
+      = ochusha_utils_get_attribute_int(pref_attrs,
+					"weight_of_response_rendering");
+  else
+    application->weight_of_response_rendering
+      = OCHUSHA_DEFAULT_WEIGHT_OF_RESPONSE_RENDERING;
+#endif
+
   if (g_hash_table_lookup(pref_attrs, "boardlist_pane_style") != NULL)
     application->boardlist_pane_style
       = ochusha_utils_get_attribute_int(pref_attrs, "boardlist_pane_style");
@@ -836,6 +985,12 @@ read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
   else
     application->popup_response_delay = OCHUSHA_DEFAULT_POPUP_DELAY;
 
+  if (g_hash_table_lookup(pref_attrs, "popup_close_delay") != NULL)
+    application->popup_close_delay
+      = ochusha_utils_get_attribute_int(pref_attrs, "popup_close_delay");
+  else
+    application->popup_close_delay = OCHUSHA_DEFAULT_POPUP_CLOSE_DELAY;
+
   if (g_hash_table_lookup(pref_attrs, "enable_smart_popup_placement") != NULL)
     application->enable_smart_popup_placement
       = ochusha_utils_get_attribute_boolean(pref_attrs,
@@ -865,22 +1020,28 @@ read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
 }
 
 
-static void
-access_started_cb(gpointer user_data)
+static gboolean
+access_started_cb(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer,
+		  OchushaApplication *application)
 {
-  IconLabel *icon_label;
   int *counter;
+  IconLabel *tab_label = g_object_get_qdata(G_OBJECT(buffer), tab_label_id);
+
 #if DEBUG_GUI
   fprintf(stderr, "access_started_cb\n");
 #endif
-  icon_label = ICON_LABEL(user_data);
-  counter = g_object_get_qdata(G_OBJECT(icon_label),
-			       tab_label_icon_counter_id);
+
+  if (tab_label == NULL)
+    return TRUE;
+
+  counter = g_object_get_qdata(G_OBJECT(tab_label), tab_label_icon_counter_id);
   *counter = 0;
 
   gdk_threads_enter();
-  icon_label_set_icon(icon_label, tab_label_icons[TAB_LABEL_ICON_PREPARATION]);
+  icon_label_set_icon(tab_label, tab_label_icons[TAB_LABEL_ICON_PREPARATION]);
   gdk_threads_leave();
+
+  return TRUE;
 }
 
 
@@ -896,92 +1057,93 @@ static int progress_icon_number[7] =
 };
 
 
-static void
-access_progressed_cb(gpointer user_data)
+static gboolean
+access_progressed_cb(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer,
+		     int bytes_read, int bytes_total,
+		     OchushaApplication *application)
 {
-  IconLabel *icon_label;
+  IconLabel *tab_label = g_object_get_qdata(G_OBJECT(buffer), tab_label_id);
   int *counter;
 #if DEBUG_GUI_MOST
   fprintf(stderr, "access_progressed_cb\n");
 #endif
 
-  if (user_data == NULL)
-    return;
+  if (tab_label == NULL)
+    return TRUE;
 
-  g_return_if_fail(IS_ICON_LABEL(user_data));
-  icon_label = ICON_LABEL(user_data);
-
-  counter = g_object_get_qdata(G_OBJECT(icon_label),
-			       tab_label_icon_counter_id);
+  counter = g_object_get_qdata(G_OBJECT(tab_label), tab_label_icon_counter_id);
   *counter = (*counter + 1) % 7;
 
   gdk_threads_enter();
-  icon_label_set_icon(icon_label,
+  icon_label_set_icon(tab_label,
 		      tab_label_icons[progress_icon_number[*counter]]);
   gdk_threads_leave();
+
+  return TRUE;
 }
 
 
-static void
-access_completed_cb(gpointer user_data)
+static gboolean
+access_completed_cb(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer,
+		    OchushaApplication *application)
 {
-  IconLabel *icon_label;
+  IconLabel *tab_label = g_object_get_qdata(G_OBJECT(buffer), tab_label_id);
 
 #if DEBUG_GUI
   fprintf(stderr, "access_completed_cb\n");
 #endif
 
-  if (user_data == NULL)
-    return;
-
-  g_return_if_fail(IS_ICON_LABEL(user_data));
-  icon_label = ICON_LABEL(user_data);
+  if (tab_label == NULL)
+    return TRUE;
 
   gdk_threads_enter();
-  icon_label_set_icon(icon_label, tab_label_icons[TAB_LABEL_ICON_DONE]);
+  icon_label_set_icon(tab_label, tab_label_icons[TAB_LABEL_ICON_DONE]);
   gdk_threads_leave();
+
+  return TRUE;
 }
 
 
-static void
-access_terminated_cb(gpointer user_data)
+static gboolean
+access_terminated_cb(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer,
+		     OchushaApplication *application)
 {
-  IconLabel *icon_label;
+  IconLabel *tab_label = g_object_get_qdata(G_OBJECT(buffer), tab_label_id);
 
 #if DEBUG_GUI
   fprintf(stderr, "access_terminated_cb\n");
 #endif
 
-  if (user_data == NULL)
-    return;
-
-  g_return_if_fail(IS_ICON_LABEL(user_data));
-  icon_label = ICON_LABEL(user_data);
+  if (tab_label == NULL)
+    return TRUE;
 
   gdk_threads_enter();
-  icon_label_set_icon(icon_label, tab_label_icons[TAB_LABEL_ICON_DONE]);
+  icon_label_set_icon(tab_label, tab_label_icons[TAB_LABEL_ICON_DONE]);
   gdk_threads_leave();
+
+  return TRUE;
 }
 
 
-static void
-access_failed_cb(gpointer user_data)
+static gboolean
+access_failed_cb(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer,
+		 OchushaNetworkBrokerFailureReason reason_code,
+		 const gchar *reason, OchushaApplication *application)
 {
-  IconLabel *icon_label;
+  IconLabel *tab_label = g_object_get_qdata(G_OBJECT(buffer), tab_label_id);
 
 #if DEBUG_GUI
   fprintf(stderr, "access_failed_cb\n");
 #endif
 
-  if (user_data == NULL)
-    return;
-
-  g_return_if_fail(IS_ICON_LABEL(user_data));
-  icon_label = ICON_LABEL(user_data);
+  if (tab_label == NULL)
+    return TRUE;
 
   gdk_threads_enter();
-  icon_label_set_icon(icon_label, tab_label_icons[TAB_LABEL_ICON_ERROR]);
+  icon_label_set_icon(tab_label, tab_label_icons[TAB_LABEL_ICON_ERROR]);
   gdk_threads_leave();
+
+  return TRUE;
 }
 
 
@@ -1122,3 +1284,144 @@ popup_open_url_dialog(OchushaApplication *application)
     }
   MAIN_UI_UNLOCK;
 }
+
+
+#if ENABLE_RENDERING_ON_IDLE
+/*
+ * ĤGDKΥƥ륻Ǽ롣
+ */
+static gint current_rendering_load = 0;
+static GSList *rendered_buffers = NULL;
+static guint advance_rendering_on_idle_func_id = 0;
+
+
+static void
+wake_up_helper(OchushaAsyncBuffer *buffer, OchushaApplication *application)
+{
+  ochusha_async_buffer_broadcast(buffer, "ochusha_ui.c: wake_up_helper()");
+  g_object_unref(G_OBJECT(buffer));
+}
+
+
+/*
+ * advance_rendering_on_idle:
+ *
+ * GLibΥᥤ롼פidleؿȤϿᥤ롼פˤˤʤä
+ * ԤäƤ󥰥åɤ򵯤
+ */
+static gboolean
+advance_rendering_on_idle(gpointer application)
+{
+  GSList *old_rendered_buffers;
+
+  gdk_threads_enter();
+
+  old_rendered_buffers = rendered_buffers;
+  rendered_buffers = NULL;
+  advance_rendering_on_idle_func_id = 0;
+  current_rendering_load = 0;
+
+  gdk_threads_leave();
+
+#if DEBUG_RENDERING_ON_IDLE
+  fprintf(stderr, "advance_rendering_on_idle()\n");
+#endif
+  /*
+   * 򵯤Τ̵̤ͥåȥξ֤
+   * ԤΥǡ̤ʤɺ٤򵤤ˤƤޤǸ̩˥åɿĴ
+   * ɬפϤʤȻפΤǡޤåɥå
+   * ƤåɤοȤΤ̵̤餯ʤΤ
+   * GUIΥ쥹ݥ󥹤˱ƶͿʤ
+   * ΤϥǤȤȤǡ
+   * application->maximum_number_of_renderingοåɤ򵯤
+   * Żʤåɤ̤Υåɤ򵯤Ȥץȥ
+   * Ȥˤ롣
+   */
+  g_slist_foreach(old_rendered_buffers, (GFunc)wake_up_helper, application);
+
+  return FALSE;
+}
+
+
+/*
+ * try_get_rendering_right:
+ *
+ * item󥰤륹åɤߥ󥰤Ԥäɤɤ
+ * Ƚꤹ롣ߥ󥰤ԤäƤ륹åɿ
+ * application->maximum_number_of_rendering_threads̤ǤС
+ * 󥰤ĤȤ̣TRUE֤ʳξˤFALSE֤
+ * 󥰤ԤåɤϤδؿFALSE֤Ȥˤϥ
+ * Ԥ鷺bufferwait롣TRUEåɤ1ñʬ󥰤
+ * Ԥä顢return_rendering_semaphore()ˤꥻޥե֤
+ *
+ * gdk_threads_enter()ĶƤӽФȡ
+ */
+gboolean
+try_get_rendering_right(OchushaApplication *application,
+			OchushaAsyncBuffer *buffer, gpointer item)
+{
+  gboolean result = FALSE;
+  PanedNotebook *contents_window
+    = PANED_NOTEBOOK(application->contents_window);
+  OchushaBulletinBoard *current_board
+    = (OchushaBulletinBoard *)paned_notebook_get_current_item(contents_window);
+
+#if DEBUG_RENDERING_ON_IDLE
+  fprintf(stderr, "try_get_rendering_semaphore(): buffer=%p, item=%p\n",
+	  buffer, item);
+#endif
+
+  if (current_board == NULL || application->rendering_unit == 0)
+    result = TRUE;
+  else
+    {
+      if (OCHUSHA_IS_BULLETIN_BOARD(item)
+	  && (current_board == item
+	      || current_rendering_load < application->rendering_unit))
+	result = TRUE;
+      else if (OCHUSHA_IS_BBS_THREAD(item))
+	{
+	  PanedNotebook *board_pane = (PanedNotebook *)paned_notebook_get_current_item_view(contents_window);
+	  if (board_pane == NULL)
+	    result = TRUE;
+	  else
+	    {
+	      OchushaBBSThread *current_thread
+		= (OchushaBBSThread *)paned_notebook_get_current_item(board_pane);
+	      if (current_thread == NULL || current_thread == item
+		  || current_rendering_load < application->rendering_unit)
+		result = TRUE;
+	    }
+	}
+    }
+
+  if (result)
+    {
+      if (OCHUSHA_IS_BULLETIN_BOARD(item))
+	current_rendering_load += application->weight_of_threadlist_rendering;
+      else if (OCHUSHA_IS_BBS_THREAD(item))
+	current_rendering_load += application->weight_of_response_rendering;
+#if DEBUG_RENDERING_ON_IDLE
+      else
+	fprintf(stderr, "Unknown item(%p) found\n", item);
+#endif
+    }
+  else
+    {
+      g_object_ref(G_OBJECT(buffer));
+      rendered_buffers = g_slist_append(rendered_buffers, buffer);
+
+      if (advance_rendering_on_idle_func_id == 0)
+	advance_rendering_on_idle_func_id
+	  = g_idle_add_full(G_PRIORITY_HIGH,
+			    advance_rendering_on_idle, application, NULL);
+    }
+
+#if DEBUG_RENDERING_ON_IDLE
+  fprintf(stderr, "  result=%d, current_rendering_load=%d\n",
+	  result, current_rendering_load);
+#endif
+
+  return result;
+}
+#endif
