@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.h,v 1.7 2003/05/19 04:15:24 fuyu Exp $
+ * $Id: ochusha_ui.h,v 1.12 2003/06/06 19:37:23 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_H_
@@ -32,7 +32,11 @@
 #include "config.h"
 #include "ochusha.h"
 #include "ochusha_private.h"
+#include "ochusha_async_buffer.h"
 #include "ochusha_bbs_table.h"
+#include "ochusha_network_broker.h"
+
+#include "icon_label.h"
 
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -67,14 +71,22 @@
 #define DEBUG_NETWORK_MOST	0
 #define DEBUG_WIDGET		0
 #define DEBUG_WIDGET_MOST	0
+#define DEBUG_RENDERING_ON_IDLE	0
 
 #define ENABLE_MAIN_TOOLBAR						0
+#define ENABLE_RENDERING_ON_IDLE					0
+
+#define ENABLE_MULTIPLE_POPUPS						1
 
 #define OCHUSHA_DEFAULT_WIDTH						600
 #define OCHUSHA_DEFAULT_HEIGHT						400
 #define OCHUSHA_DEFAULT_BOARDLIST_WIDTH					150
 #define OCHUSHA_DEFAULT_THREADLIST_HEIGHT				100
 
+#define OCHUSHA_DEFAULT_RENDERING_UNIT					300
+#define OCHUSHA_DEFAULT_WEIGHT_OF_THREADLIST_RENDERING			10
+#define OCHUSHA_DEFAULT_WEIGHT_OF_RESPONSE_RENDERING			20
+
 #define OCHUSHA_DEFAULT_BOARD_TAB_SHRINKABLE				FALSE
 #define OCHUSHA_DEFAULT_BOARD_TAB_MINIMUM_SIZE				80
 #define OCHUSHA_DEFAULT_BOARD_TAB_ENABLE_TOOLTIPS			FALSE
@@ -85,6 +97,7 @@
 #define OCHUSHA_DEFAULT_SHOW_THREADLIST_ENTRY_WHEN_THREAD_SELECTED	FALSE
 
 #define OCHUSHA_DEFAULT_POPUP_DELAY					500
+#define OCHUSHA_DEFAULT_POPUP_CLOSE_DELAY				100
 #define OCHUSHA_DEFAULT_MAXIMUM_NUMBER_OF_POPUP_RESPONSES		5
 
 
@@ -151,6 +164,7 @@ struct _OchushaApplication
   /* İθǡ */
   OchushaBBSTable *table;
 
+
   /* windowʤ */
   gint width;
   gint height;
@@ -158,6 +172,14 @@ struct _OchushaApplication
   gint threadlist_height;
 
 
+#if ENABLE_RENDERING_ON_IDLE	/* ݤ */
+  /* åȤΥ٤Ĵ */
+  gint rendering_unit;
+  gint weight_of_threadlist_rendering;
+  gint weight_of_response_rendering;
+#endif
+
+
   /* paneɽˡʤ */
   gint boardlist_pane_style;
   gboolean board_tab_shrinkable;
@@ -179,6 +201,7 @@ struct _OchushaApplication
   /* 쥹Υݥåץå״Ϣ */
   gboolean enable_popup_response;
   guint popup_response_delay;
+  guint popup_close_delay;
   gboolean enable_smart_popup_placement;
   gint maximum_number_of_popup_responses;
 
@@ -190,7 +213,9 @@ struct _OchushaApplication
   /* Υ桼ǥեȥե륿롼 */
   ThreadlistFilter user_default_filter;
 
-  OchushaConfig conf;
+  OchushaNetworkBroker *broker;
+
+  OchushaConfig config;
 };
 
 
@@ -207,11 +232,6 @@ typedef enum
 } OchushaMainMenuAction;
 
 
-#ifndef OCHUSHA_PRIVATE
-extern OchushaNetworkCallbacks icon_indicator_funcs;
-#endif
-
-
 void ochusha_init_application(OchushaApplication *application);
 void ochusha_set_main_menu_sensitive(OchushaMainMenuAction action,
 				     gboolean sensitive);
@@ -226,8 +246,15 @@ void set_current_url(OchushaApplication *application, const gchar *url);
 const gchar *get_current_url(OchushaApplication *application);
 #endif
 
+void setup_for_tab_label_animation(OchushaAsyncBuffer *buffer,
+				   IconLabel *tab_label);
 GtkWidget *get_tab_label(const gchar *label_text);
-void recycle_tab_label(GtkWidget *tab_label);
+
+#if ENABLE_RENDERING_ON_IDLE
+gboolean try_get_rendering_right(OchushaApplication *application,
+				 OchushaAsyncBuffer *buffer,
+				 gpointer item);
+#endif
 
 
 #ifdef __cplusplus
