@@ -23,16 +23,16 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_view.c,v 1.9 2003/05/17 08:14:47 fuyu Exp $
+ * $Id: threadlist_view.c,v 1.13 2003/06/07 14:35:31 fuyu Exp $
  */
 
 #include "config.h"
 
+#include "ochusha_private.h"
 #include "ochusha.h"
-#include "ochusha_ui.h"
-
 #include "ochusha_bbs_thread.h"
 
+#include "ochusha_ui.h"
 #include "threadlist_view.h"
 
 #include "marshal.h"
@@ -76,7 +76,7 @@ static gboolean tree_view_motion_notify_event_cb(GtkWidget *widget,
 						 GdkEventMotion *event,
 						 ThreadlistView *view);
 static gboolean tree_view_leave_notify_event_cb(GtkWidget *widget,
-						GdkEventButton *event,
+						GdkEventCrossing *event,
 						ThreadlistView *view);
 static gboolean tree_view_button_press_event_cb(GtkWidget *widget,
 						GdkEventButton *event,
@@ -724,6 +724,18 @@ threadlist_view_destroy(GtkObject *object)
 }
 
 
+#if TRACE_MEMORY_USAGE
+static void
+trace_free(gpointer pointer)
+{
+  G_FREE(pointer);
+}
+#define TRACE_FREE	trace_free
+#else
+#define TRACE_FREE	g_free
+#endif
+
+
 static ThreadlistInfo *
 ensure_threadlist_info(OchushaBBSThread *thread)
 {
@@ -731,9 +743,9 @@ ensure_threadlist_info(OchushaBBSThread *thread)
 					    list_entry_info_id);
   if (info == NULL)
     {
-      info = g_new0(ThreadlistInfo, 1);
+      info = G_NEW0(ThreadlistInfo, 1);
       g_object_set_qdata_full(G_OBJECT(thread), list_entry_info_id,
-			      info, (GDestroyNotify)g_free);
+			      info, (GDestroyNotify)TRACE_FREE);
     }
 
   return info;
@@ -754,14 +766,14 @@ threadlist_view_title_not_equals(GtkTreeModel *model, gint column,
 #if DEBUG_SEARCH
   char *native_string;
 #endif
-  /* XXX */
+
 #if DEBUG_SEARCH
   if (key != NULL)
     {
       native_string = convert_string(utf8_to_native, key, -1);
       fprintf(stderr, "threadlist_view_title_not_equals(): column=%d, key=\"%s\":",
 	      column, key);
-      free(native_string);
+      G_FREE(native_string);
     }
 #endif
   if (key == NULL)
@@ -773,7 +785,7 @@ threadlist_view_title_not_equals(GtkTreeModel *model, gint column,
     {
       native_string = convert_string(utf8_to_native, title, -1);
       fprintf(stderr, " title at iter=\"%s\"\n", native_string);
-      free(native_string);
+      G_FREE(native_string);
     }
   else
     fprintf(stderr, " title at iter=NULL\n");
@@ -1258,8 +1270,8 @@ popup_menu_callback(gpointer data, guint action, GtkWidget *widget)
 {
   ThreadlistView *view = gtk_item_factory_popup_data(popup_menu_item_factory);
 #if DEBUG_GUI_MOST
-  fprintf(stderr, "popup_menu_callback: view=0x%x, action=%d\n",
-	  (int)view, action);
+  fprintf(stderr, "popup_menu_callback: view=%p, action=%d\n",
+	  view, action);
 #endif
   threadlist_view_do_on_selection(view, action);
 }
@@ -1270,7 +1282,7 @@ popup_menu_destructed_cb(gpointer data)
 {
 #if DEBUG_GUI_MOST
   ThreadlistView *view = THREADLIST_VIEW(data);
-  fprintf(stderr, "popup_menu_destructed_cb: data=0x%x\n", (int)view);
+  fprintf(stderr, "popup_menu_destructed_cb: data=%p\n", view);
 #endif
 }
 
@@ -1328,7 +1340,7 @@ tree_view_motion_notify_event_cb(GtkWidget *widget, GdkEventMotion *event,
 
 
 static gboolean
-tree_view_leave_notify_event_cb(GtkWidget *widget, GdkEventButton *event,
+tree_view_leave_notify_event_cb(GtkWidget *widget, GdkEventCrossing *event,
 				ThreadlistView *view)
 {
   if (view->recently_pointed_thread != NULL)
@@ -1524,8 +1536,8 @@ threadlist_view_update_thread_visual(ThreadlistView *view,
   if (info->threadlist_view != view)
     {
 #if DEBUG_WIDGET
-      fprintf(stderr, "info->threadlist_view=0x%x, view=0x%x\n",
-	      (int)info->threadlist_view, (int)view);
+      fprintf(stderr, "info->threadlist_view=%p, view=%p\n",
+	      info->threadlist_view, view);
 #endif
       return;
     }
@@ -1707,6 +1719,10 @@ threadlist_view_close(ThreadlistView *view, OchushaBBSThread *last_thread)
   g_return_if_fail(IS_THREADLIST_VIEW(view));
 
   view->model_is_opened = FALSE;
+
+  if (last_thread == NULL)
+    return;
+
   info = ensure_threadlist_info(last_thread);
   g_return_if_fail(info != NULL
 		   && gtk_list_store_iter_is_valid(GTK_LIST_STORE(view->model),
@@ -1749,8 +1765,8 @@ threadlist_view_update_thread(ThreadlistView *view, OchushaBBSThread *thread)
   if (info->threadlist_view != view)
     {
 #if DEBUG_WIDGET	/* ΥƥȼΤĹ */
-      fprintf(stderr, "info->threadlist_view=0x%x, view=0x%x\n",
-	      (int)info->threadlist_view, (int)view);
+      fprintf(stderr, "info->threadlist_view=%p, view=%p\n",
+	      info->threadlist_view, view);
 #endif
       return;
     }
