@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_view.h,v 1.2 2003/05/11 20:55:12 fuyu Exp $
+ * $Id: threadlist_view.h,v 1.6 2003/05/17 08:14:47 fuyu Exp $
  */
 
 #ifndef _THREADLIST_VIEW_H_
@@ -52,6 +52,21 @@ typedef struct _ThreadlistViewClass ThreadlistViewClass;
 typedef struct _ThreadlistInfo ThreadlistInfo;
 
 
+typedef enum {
+  THREADLIST_SEARCH_DIRECTION_BACKWARD,
+  THREADLIST_SEARCH_DIRECTION_FORWARD
+} ThreadlistSearchDirection;
+
+
+typedef enum {
+  THREADLIST_SEARCH_ACTION_FORWARD,
+  THREADLIST_SEARCH_ACTION_BACKWARD,
+  THREADLIST_SEARCH_ACTION_REGEXP_FORWARD,
+  THREADLIST_SEARCH_ACTION_REGEXP_BACKWARD,
+  THREADLIST_SEARCH_ACTION_TERMINATE
+} ThreadlistSearchAction;
+
+
 struct _ThreadlistView
 {
   GtkScrolledWindow container;
@@ -68,15 +83,19 @@ struct _ThreadlistView
 
   GtkTreeSelection *selection;
 
+
+  /* Ϣ */
+  gboolean (*title_match_func)(const gchar *title, gpointer search_data);
+  gint last_search_position;
+  GtkTreePath *last_path;
+
   gint last_rank_diff;
 
-  gint number_of_threads;
-
-  ThreadlistInfo *(*threadlist_info_new)(OchushaBBSThread *thread);
-
   OchushaBBSThread *recently_pointed_thread;
 
-  gboolean model_is_opened;
+  guint model_is_opened		: 1;
+  guint use_native_search	: 1;
+  guint being_searched		: 1;
 };
 
 
@@ -105,6 +124,8 @@ struct _ThreadlistViewClass
 
   gboolean (*move_cursor)(ThreadlistView *threadlist_view,
 			  GtkMovementStep step, gint count);
+  gboolean (*interactive_search)(ThreadlistView *threadlist_view,
+				 ThreadlistSearchAction search_action);
 
   void (*do_on_selection)(ThreadlistView *threadlist_view, gint what_to_do);
   void (*do_on_thread_at_cursor)(ThreadlistView *threadlist_view,
@@ -112,45 +133,99 @@ struct _ThreadlistViewClass
 };
 
 
-struct _ThreadlistInfo
+/*
+ * ˤ륨ȥθɤѥ᡼
+ */
+typedef enum
 {
-  GtkTreeIter iter;
-  ThreadlistView *threadlist_view;
+  THREADLIST_ENTRY_FLAG_MARK = 		1 << 0,		/* ˥å */
+  THREADLIST_ENTRY_FLAG_STRIKE = 	1 << 1,		/* ä */
+} ThreadlistEntryVisualFlags;
 
-  /* Ǹ˥򹹿Ȥν */
-  gint view_rank;
 
-  /* ˤ(ThreadlistEntryStateORȤä) */
-  gint view_flags;
-};
+typedef struct _ThreadlistEntryVisualParameter
+{
+  guint flags			: 2;	/* ThreadlistEntryVisualFlagsOR */
+
+  guint foreground_color	: 4;	/* 16¿뤯餤 */
+  guint background_color	: 4;	/* 16¿뤯餤 */
+
+  guint reserved		: 22;
+} ThreadlistEntryVisualParameter;
+
+
+#define THREADLIST_ENTRY_VISUAL_FLAGS_WIDTH		2
+#define THREADLIST_ENTRY_VISUAL_FOREGROUND_COLOR_WIDTH	4
+#define THREADLIST_ENTRY_VISUAL_BACKGROUND_COLOR_WIDTH	4
+
+
+/*
+ * ʲϤäỌ̇̄Τ̾ˤ٤桼ǽˤ뤳Ȥ
+ * ͤʤ顢ϤΥåȤGUIԤ¦Ƿ
+ * ٤
+ *
+ * ġĤȸ櫓ǰŪȤȤˤƤ
+ */
+typedef enum
+{
+  LIST_ENTRY_FOREGROUND_COLOR_NORMAL = 0,
+  LIST_ENTRY_FOREGROUND_COLOR_BLUE = 1,
+  LIST_ENTRY_FOREGROUND_COLOR_RED = 2,
+  LIST_ENTRY_FOREGROUND_COLOR_GREEN = 3,
+  LIST_ENTRY_FOREGROUND_COLOR_GRAY50 = 4,
+  LIST_ENTRY_FOREGROUND_COLOR_GRAY75 = 5,
+} ThreadlistEntryForegroundColor;
 
 
 typedef enum
 {
-  BBS_THREAD_FAVORITE = 1 << 0,
-  BBS_THREAD_HIDDEN = 1 << 1,
-  BBS_THREAD_HIGHLIGHT = 1 << 2,
-} ThreadlistEntryState;
+  LIST_ENTRY_BACKGROUND_COLOR_NORMAL = 0,
+  LIST_ENTRY_BACKGROUND_COLOR_LAVENDER = 1,
+  LIST_ENTRY_BACKGROUND_COLOR_PINK = 2,
+  LIST_ENTRY_BACKGROUND_COLOR_DARK_SEA_GREEN1 = 3,
+  LIST_ENTRY_BACKGROUND_COLOR_GRAY75 = 4,
+  LIST_ENTRY_BACKGROUND_COLOR_GRAY90 = 5,
+} ThreadlistEntryBackgroundColor;
 
 
 GType threadlist_view_get_type(void);
 GtkWidget *threadlist_view_new(void);
 
 
-typedef ThreadlistInfo *ThreadlistInfoNewFunc(OchushaBBSThread *thread);
-
-gboolean threadlist_view_open(ThreadlistView *view,
-			      ThreadlistInfoNewFunc *factory);
-void threadlist_view_close(ThreadlistView *view);
+gboolean threadlist_view_open(ThreadlistView *view);
+void threadlist_view_close(ThreadlistView *view,
+			   OchushaBBSThread *last_thread);
 void threadlist_view_append_thread(ThreadlistView *view,
-				   OchushaBBSThread *thread, gint rank);
+				   OchushaBBSThread *thread,
+				   gint last_rank, gint new_rank);
 void threadlist_view_update_thread(ThreadlistView *view,
 				   OchushaBBSThread *thread);
+void threadlist_view_update_thread_visual(ThreadlistView *view,
+				OchushaBBSThread *thread,
+				ThreadlistEntryVisualParameter *parameter);
+
 void threadlist_view_scroll_to_thread(ThreadlistView *view,
 				      OchushaBBSThread *thread);
 
+void threadlist_view_set_enable_native_search(ThreadlistView *view,
+					      gboolean enable);
+gboolean threadlist_view_get_enable_native_search(ThreadlistView *view);
+
 gboolean threadlist_view_start_interactive_search(ThreadlistView *view);
 
+typedef gboolean (ThreadlistViewTitleMatchFunc)(const gchar *title,
+						gpointer search_data);
+
+void threadlist_view_set_title_match_func(ThreadlistView *view,
+					  ThreadlistViewTitleMatchFunc *func);
+ThreadlistViewTitleMatchFunc *threadlist_view_get_title_match_func(
+							ThreadlistView *view);
+void threadlist_view_set_being_searched(ThreadlistView *view, gboolean being);
+gboolean threadlist_view_find_next(ThreadlistView *view,
+				   ThreadlistSearchDirection direction,
+				   gboolean enable_wrap,
+				   gboolean include_current_pos,
+				   gpointer search_data);
 #ifdef __cplusplus
 } // extern "C"
 #endif
