@@ -23,10 +23,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: gziputils.c,v 1.1.1.1 2003/05/10 16:34:27 fuyu Exp $
+ * $Id: gziputils.c,v 1.2 2003/06/06 19:38:04 fuyu Exp $
  */
 
+#include "config.h"
+
+#include "ochusha_private.h"
 #include "ochusha_async_buffer.h"
+
 #include "gziputils.h"
 
 #include <stdio.h>
@@ -61,9 +65,7 @@ struct _GzipBuffer
 GzipBuffer *
 gzip_buffer_new(OchushaAsyncBuffer *async_buffer)
 {
-  GzipBuffer *buffer = (GzipBuffer *)calloc(1, sizeof(GzipBuffer));
-  if (buffer == NULL)
-    return NULL;
+  GzipBuffer *buffer = G_NEW0(GzipBuffer, 1);
 
   buffer->output = async_buffer;
   buffer->status = GZIP_BUFFER_INITIAL;
@@ -81,7 +83,7 @@ gzip_buffer_new(OchushaAsyncBuffer *async_buffer)
 #if DEBUG_GZIP
       fprintf(stderr, "inflateInit2() failed.\n");
 #endif
-      free(buffer);
+      G_FREE(buffer);
       return NULL;
     }
 
@@ -94,8 +96,8 @@ gzip_buffer_free(GzipBuffer *buffer)
 {
   inflateEnd(&buffer->zstream);
   if (buffer->pending_buffer != NULL)
-    free(buffer->pending_buffer);
-  free(buffer);
+    G_FREE(buffer->pending_buffer);
+  G_FREE(buffer);
 }
 
 
@@ -121,18 +123,10 @@ gzip_buffer_append_data(GzipBuffer *buffer, const char *data,
       char *new_buf;
       int pending_length = buffer->pending_length + length;
       if (buffer->pending_buffer_length < pending_length)
-	new_buf = (char *)realloc(buffer->pending_buffer, pending_length);
+	new_buf = (char *)G_REALLOC(buffer->pending_buffer, pending_length);
       else
 	new_buf = buffer->pending_buffer;
 
-      if (new_buf == NULL)
-	{
-#if DEBUG_GZIP
-	  fprintf(stderr, "Out of memory.\n");
-#endif
-	  buffer->status = GZIP_BUFFER_ERROR;
-	  return buffer->status;
-	}
       memcpy(new_buf + buffer->pending_length, data, length);
       length = pending_length;
       data = new_buf;
@@ -272,17 +266,8 @@ inflate_gzip_buffer(const char *input, unsigned int input_length,
 
   if (out_buf == NULL || out_len < input_length * 2)
     {
-      char *new_buf;
       out_len = input_length * 2;	/* 褺Ϥܽ */
-      new_buf = (char *)realloc(out_buf, out_len);
-      if (new_buf == NULL)
-	{
-#if DEBUG_GZIP
-	  fprintf(stderr, "inflation failed: out of memory.\n");
-#endif
-	  return FALSE;
-	}
-      out_buf = new_buf;
+      out_buf = (char *)G_REALLOC(out_buf, out_len);
     }
 
   zstream.next_out = out_buf;
@@ -294,17 +279,8 @@ inflate_gzip_buffer(const char *input, unsigned int input_length,
 	{
 	case Z_STREAM_END:
 	  {
-	    char *final_buf;
 	    inflateEnd(&zstream);
-	    final_buf = (char *)realloc(out_buf, zstream.total_out);
-	    if (final_buf == NULL)
-	      {
-		final_buf = out_buf;
-#if DEBUG_GZIP
-		fprintf(stderr, "Couldn't shorten buffer.\n");
-#endif
-	      }
-	    *output = final_buf;
+	    *output = (char *)G_REALLOC(out_buf, zstream.total_out);
 	    *output_length = zstream.total_out;
 	    return TRUE;
 	  }
@@ -316,18 +292,9 @@ inflate_gzip_buffer(const char *input, unsigned int input_length,
 	case Z_OK:
 	  /* output buffer full */
 	  {
-	    char *new_out_buf;
 	    out_len *= 2;	/*  */
-	    new_out_buf = (char *)realloc(out_buf, out_len);
-	    if (new_out_buf == NULL)
-	      {
-#if DEBUG_GZIP
-		fprintf(stderr, "inflation failed: out of memory.\n");
-#endif
-		goto inflation_failed;
-	      }
-	    out_buf = new_out_buf;
-	    zstream.next_out = new_out_buf + zstream.total_out;
+	    out_buf = (char *)G_REALLOC(out_buf, out_len);
+	    zstream.next_out = out_buf + zstream.total_out;
 	    zstream.avail_out += out_len / 2;
 	    break;
 	  }
@@ -344,7 +311,7 @@ inflate_gzip_buffer(const char *input, unsigned int input_length,
   inflateEnd(&zstream);
 
   if (*output == NULL)
-    free(out_buf);	/* ߤout_bufϤδؿǿ˳ݤ */
+    G_FREE(out_buf);	/* ߤout_bufϤδؿǿ˳ݤ */
   else
     *output = out_buf;	/* reallocΤʤcallerˤޤ */
 
