@@ -23,11 +23,13 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: htmlutils.c,v 1.1.1.1 2003/05/10 16:34:27 fuyu Exp $
+ * $Id: htmlutils.c,v 1.2 2003/06/06 19:38:04 fuyu Exp $
  */
 
 #include "config.h"
 
+#include "ochusha_private.h"
+
 #include "htmlutils.h"
 #include "utils.h"
 
@@ -433,7 +435,7 @@ default_entity_ref_handler(ElementHandler *handler, void *context,
 
   (*handler->characters)(context, buffer, strlen(buffer));
   if (utf8_entity != NULL)
-    free(utf8_entity);
+    G_FREE(utf8_entity);
 
   return TRUE;
 }
@@ -535,29 +537,19 @@ parse_text(ElementHandler *handler, void *context, const char *text, int len)
 		    {
 		      if (errno == E2BIG)
 			{
-			  gchar *old_buffer = buffer;
 			  size_t old_size = buffer_size;
 			  /* allocate 2 times larger buffer in the heap. */
 			  buffer_size *= 2;
 			  if (buffer != default_buffer)
-			    buffer = (gchar *)realloc(buffer, buffer_size);
+			    buffer = (gchar *)G_REALLOC(buffer, buffer_size);
 			  else
 			    {
-			      buffer = (gchar *)malloc(buffer_size);
+			      gchar *old_buffer = buffer;
+			      buffer = (gchar *)G_MALLOC(buffer_size);
 			      if (buffer != NULL)
 				memcpy(buffer, old_buffer,
 				       old_size - outbytesleft);
 			    }
-
-			  if (buffer == NULL)
-			    {
-			      if (old_buffer != default_buffer)
-				free(old_buffer);
-
-			      fprintf(stderr,
-				      "iconv failed: no sufficient space.\n");
-			      return NULL;
-			    }
 			  outbytesleft += old_size;
 			  outbuf = buffer + buffer_size - outbytesleft;
 			}
@@ -580,7 +572,7 @@ parse_text(ElementHandler *handler, void *context, const char *text, int len)
 			  fprintf(stderr, "\n");
 #endif
 			  if (buffer != default_buffer)
-			    free(buffer);
+			    G_FREE(buffer);
 #if DEBUG_ICONV
 			  abort();
 #else
@@ -667,7 +659,7 @@ process_tag(ElementHandler *handler, void *context, const char *head, int len)
   else
     end_tag = FALSE;
 
-  name = g_strndup(cur_pos, tmp_pos - cur_pos);
+  name = G_STRNDUP(cur_pos, tmp_pos - cur_pos);
   len -= tmp_pos - cur_pos;	/* ̾ʬ򺹤 */
   cur_pos = tmp_pos;
 
@@ -696,7 +688,7 @@ process_tag(ElementHandler *handler, void *context, const char *head, int len)
       /* λcur_posϥȥӥ塼̾ƬؤƤϤ */
       if (!buffer_full)
 	{
-	  attrs[n_attrs * 2] = g_strndup(cur_pos, tmp_pos - cur_pos);
+	  attrs[n_attrs * 2] = G_STRNDUP(cur_pos, tmp_pos - cur_pos);
 	  n_attrs++;
 	}
 
@@ -725,7 +717,7 @@ process_tag(ElementHandler *handler, void *context, const char *head, int len)
 		  goto finish;
 		}
 	      if (!buffer_full)
-		attrs[n_attrs * 2 - 1] = g_strndup(cur_pos, tmp_pos - cur_pos);
+		attrs[n_attrs * 2 - 1] = G_STRNDUP(cur_pos, tmp_pos - cur_pos);
 	      len -= (tmp_pos - cur_pos + 1);
 	      cur_pos = tmp_pos + 1;
 	    }
@@ -740,7 +732,7 @@ process_tag(ElementHandler *handler, void *context, const char *head, int len)
 		  goto finish;
 		}
 	      if (!buffer_full)
-		attrs[n_attrs * 2 - 1] = g_strndup(cur_pos, tmp_pos - cur_pos);
+		attrs[n_attrs * 2 - 1] = G_STRNDUP(cur_pos, tmp_pos - cur_pos);
 	      len -= (tmp_pos - cur_pos);
 	      cur_pos = tmp_pos;
 	    }
@@ -772,13 +764,13 @@ process_tag(ElementHandler *handler, void *context, const char *head, int len)
     }
 
  finish:
-  free(name);
+  G_FREE(name);
 
   while (--n_attrs >= 0)
     {
-      free(attrs[n_attrs * 2]);
+      G_FREE(attrs[n_attrs * 2]);
       if (attrs[n_attrs * 2 + 1] != NULL)
-	free(attrs[n_attrs * 2 + 1]);
+	G_FREE(attrs[n_attrs * 2 + 1]);
     }
 
   return (cur_pos != NULL) ? cur_pos + 1 : NULL;
@@ -845,7 +837,7 @@ process_entity_ref(ElementHandler *handler, void *context, const char *head,
     return NULL;	/* ¿ʬ */
 
   if (size >= DEFAULT_ENTITY_SIZE)
-    entity = g_strndup(head + 1, size);
+    entity = G_STRNDUP(head + 1, size);
   else
     {
       entity = default_buffer;
@@ -875,14 +867,14 @@ process_entity_ref(ElementHandler *handler, void *context, const char *head,
 	(*handler->characters)(context, broken_ref_buffer,
 			       strlen(broken_ref_buffer));
 	if (utf8_entity != NULL)
-	  free(utf8_entity);
+	  G_FREE(utf8_entity);
 #if DEBUG_PARSER_MOST
 	fprintf(stderr, "broken entity ref: %s -> %s\n", entity, broken_ref_buffer);
 #endif
       }
 
   if (entity != default_buffer)
-    free(entity);
+    G_FREE(entity);
 
   return ref_tail + (*ref_tail == ';' ? 1 : 0);
 }
@@ -1071,23 +1063,12 @@ append_characters(void *ctx, const gchar *ch, int len)
       gchar *new_buffer;
       if (context->buf_len == DEFAULT_BUFFER_SIZE)
 	{
-	  new_buffer = (gchar *)malloc(context->buf_len * 2);
-	  if (new_buffer == NULL)
-	    {
-	      context->give_up = TRUE;
-	      return;	/*  */
-	    }
+	  new_buffer = (gchar *)G_MALLOC(context->buf_len * 2);
 	  memcpy(new_buffer, context->buffer, context->number_of_chars);
 	}
       else
-	{
-	  new_buffer = (gchar *)realloc(context->buffer, context->buf_len * 2);
-	  if (new_buffer == NULL)
-	    {
-	      context->give_up = TRUE;
-	      return;	/*  */
-	    }
-	}
+	new_buffer = (gchar *)G_REALLOC(context->buffer, context->buf_len * 2);
+
       context->buf_len *= 2;
       context->buffer = new_buffer;
     }
@@ -1141,7 +1122,7 @@ simple_string_canon(const char *src, int len, iconv_t converter)
     }
 
   if (context.buffer == buffer)
-    return strdup(context.buffer);
+    return G_STRDUP(context.buffer);
 
   return context.buffer;
 }
