@@ -23,11 +23,13 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_table.c,v 1.1.1.1 2003/05/10 16:34:28 fuyu Exp $
+ * $Id: ochusha_bbs_table.c,v 1.4 2003/06/07 14:35:31 fuyu Exp $
  */
 
-#include "ochusha.h"
+#include "config.h"
+
 #include "ochusha_private.h"
+#include "ochusha.h"
 #include "ochusha_bbs_table.h"
 #include "ochusha_board_category.h"
 #include "ochusha_bulletin_board.h"
@@ -219,7 +221,7 @@ ochusha_bbs_table_add_category(OchushaBBSTable *table,
   g_return_if_fail(OCHUSHA_IS_BBS_TABLE(table)
 		   && OCHUSHA_IS_BOARD_CATEGORY(category));
 
-  if (g_hash_table_lookup(table->category_table, category->name))
+  if (g_hash_table_lookup(table->category_table, category->name) != NULL)
     return;	/* ϿѤξ */
 
   g_object_ref(G_OBJECT(category));
@@ -355,13 +357,13 @@ cleanup_sax_context(SAXContext *context)
 {
   if (context->current_attr_name != NULL)
     {
-      g_free(context->current_attr_name);
+      G_FREE(context->current_attr_name);
       context->current_attr_name = NULL;
     }
 
   if (context->current_attr_val != NULL)
     {
-      g_free(context->current_attr_val);
+      G_FREE(context->current_attr_val);
       context->current_attr_val = NULL;
     }
 
@@ -385,6 +387,18 @@ cleanup_sax_context(SAXContext *context)
 }
 
 
+#if TRACE_MEMORY_USAGE
+static void
+trace_free(gpointer pointer)
+{
+  G_FREE(pointer);
+}
+#define TRACE_FREE	trace_free
+#else
+#define TRACE_FREE	g_free
+#endif
+
+
 static void
 startElementHandler(void *context, const xmlChar *name, const xmlChar **atts)
 {
@@ -401,9 +415,11 @@ startElementHandler(void *context, const xmlChar *name, const xmlChar **atts)
       if (strcmp(name, "boardlist") == 0)
 	{
 	  sax_context->category_attributes
-	    = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	    = g_hash_table_new_full(g_str_hash, g_str_equal,
+				    TRACE_FREE, TRACE_FREE);
 	  sax_context->board_attributes
-	    = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	    = g_hash_table_new_full(g_str_hash, g_str_equal,
+				    TRACE_FREE, TRACE_FREE);
 	  sax_context->state = SAX_BOARDLIST;
 	  return;
 	}
@@ -435,13 +451,13 @@ startElementHandler(void *context, const xmlChar *name, const xmlChar **atts)
 		      sax_context->current_attr_name,
 		      sax_context->current_attr_val);
 #endif
-	      free(sax_context->current_attr_name);
-	      free(sax_context->current_attr_val);
+	      G_FREE(sax_context->current_attr_name);
+	      G_FREE(sax_context->current_attr_val);
 	      sax_context->current_attr_name = NULL;
 	      sax_context->current_attr_val = NULL;
 	      break;	/* 顼 */
 	    }
-	  sax_context->current_attr_name = strdup(atts[1]);
+	  sax_context->current_attr_name = G_STRDUP(atts[1]);
 	  return;
 	}
       else if (sax_context->state == SAX_CATEGORY
@@ -506,10 +522,10 @@ startElementHandler(void *context, const xmlChar *name, const xmlChar **atts)
 		      sax_context->current_attr_name,
 		      sax_context->current_attr_val, atts[1]);
 #endif
-	      g_free(sax_context->current_attr_val);
+	      G_FREE(sax_context->current_attr_val);
 	    }
 
-	  sax_context->current_attr_val = strdup(atts[1]);
+	  sax_context->current_attr_val = G_STRDUP(atts[1]);
 	  return;
 	}
       else if (strcmp(name, "string") == 0)
@@ -651,7 +667,7 @@ endElementHandler(void *context, const xmlChar *name)
 	   * ƤSAXѡŪʣǤȤƸƤޤ礬ꡢ
 	   * Ϣ뤹뤿ΰХåեɬפǤ뤫顣
 	   */
-	  sax_context->current_attr_val = strdup(sax_context->current_attr_val);
+	  sax_context->current_attr_val = G_STRDUP(sax_context->current_attr_val);
 	  return;
 	}
       break;	/* 顼 */
@@ -686,6 +702,8 @@ endElementHandler(void *context, const xmlChar *name)
 
 	      if (category == NULL)
 		category = ochusha_board_category_new(category_name);
+	      else
+		g_object_ref(G_OBJECT(category));
 
 #if DEBUG_LIBOCHUSHA_MORE
 	      fprintf(stderr, "emitting read_boardlist_element for category.\n");
@@ -706,6 +724,7 @@ endElementHandler(void *context, const xmlChar *name)
 						   sax_context->board_list);
 	      sax_context->board_list = NULL;
 	      ochusha_bbs_table_add_category(table, category);
+	      g_object_unref(G_OBJECT(category));
 #if DEBUG_SAX_HANDLER
 	      fprintf(stderr, "adding category: %s\n", category_name);
 #endif
@@ -782,6 +801,7 @@ endElementHandler(void *context, const xmlChar *name)
 				sax_context->board_attributes);
 
 		  ochusha_bbs_table_add_board(table, board);
+		  g_object_unref(G_OBJECT(board));
 		}
 	    }
 
@@ -864,7 +884,7 @@ getEntityHandler(void *context, const xmlChar *name)
 
 gboolean
 ochusha_bbs_table_read_boardlist_xml(OchushaBBSTable *table,
-				     OchushaConfig *conf)
+				     OchushaConfig *config)
 {
   SAXContext context =
     {
@@ -877,9 +897,9 @@ ochusha_bbs_table_read_boardlist_xml(OchushaBBSTable *table,
     };
   xmlSAXHandler sax_handler;
   char *pathname;
-  g_return_val_if_fail(OCHUSHA_IS_BBS_TABLE(table) && conf != NULL, FALSE);
+  g_return_val_if_fail(OCHUSHA_IS_BBS_TABLE(table) && config != NULL, FALSE);
 
-  pathname = ochusha_config_find_file(conf, OCHUSHA_BOARDLIST_XML);
+  pathname = ochusha_config_find_file(config, OCHUSHA_BOARDLIST_XML);
   if (pathname == NULL)
     return FALSE;
 
@@ -903,7 +923,7 @@ ochusha_bbs_table_read_boardlist_xml(OchushaBBSTable *table,
 
   if (context.state == SAX_ACCEPTED)
     {
-      g_free(pathname);
+      G_FREE(pathname);
       return TRUE;
     }
 
@@ -932,8 +952,8 @@ write_board(gpointer data, gpointer user_data)
 			"write_boardlist_element",
 			boardlist_xml);
 #if DEBUG_LIBOCHUSHA_MORE
-  fprintf(stderr, "table=0x%x, board=0x%x, boardlist_xml=0x%x\n",
-	  (int)args->table, (int)board, (int)boardlist_xml);
+  fprintf(stderr, "table=%p, board=%p, boardlist_xml=%p\n",
+	  args->table, board, boardlist_xml);
 #endif
   g_signal_emit(G_OBJECT(args->table),
 		bbs_table_signals[BOARDLIST_WRITE_BOARD_ELEMENT_SIGNAL],
