@@ -23,17 +23,19 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.c,v 1.3 2003/05/14 17:14:19 fuyu Exp $
+ * $Id: ochusha_bbs_thread.c,v 1.6 2003/06/07 14:35:32 fuyu Exp $
  */
 
+#include "config.h"
 
 #include "ochusha_private.h"
 #include "ochusha_bbs_thread.h"
 #include "ochusha_bulletin_board.h"
 
+#include "utils.h"
+
 #include "marshal.h"
 
-#include <glib-object.h>
 #include <glib.h>
 
 #include <stdio.h>
@@ -164,10 +166,10 @@ ochusha_bbs_thread_class_init(OchushaBBSThreadClass *klass)
   klass->write_threadlist_element
     = ochusha_bbs_thread_write_threadlist_element;
 
-  klass->get_response_character_encoding = NULL;
   klass->get_responses_source = NULL;
   klass->parse_responses = NULL;
-  klass->get_thread_url = NULL;
+  klass->get_url = NULL;
+  klass->get_url_to_post = NULL;
   klass->check_url = NULL;
 }
 
@@ -195,19 +197,21 @@ ochusha_bbs_thread_finalize(GObject *object)
 
   if (thread->board != NULL)
     {
+#if 0	/* QƂ[v̂ŉ^pŃJo[ */
       g_object_unref(thread->board);
+#endif
       thread->board = NULL;
     }
 
   if (thread->id != NULL)
     {
-      g_free(thread->id);
+      G_FREE(thread->id);
       thread->id = NULL;
     }
 
   if (thread->title != NULL)
     {
-      g_free(thread->title);
+      G_FREE(thread->title);
       thread->title = NULL;
     }
 
@@ -292,11 +296,13 @@ ochusha_bbs_thread_set_board(OchushaBBSThread *thread,
   if (thread->board == board)
     return;
 
+#if 0	/* QƂ[vĂ܂̂ŉ^pŃJo[ */
   if (thread->board != NULL)
     g_object_unref(G_OBJECT(thread->board));
 
   if (board != NULL)
     g_object_ref(G_OBJECT(board));
+#endif
 
   thread->board = board;
 
@@ -319,8 +325,8 @@ ochusha_bbs_thread_set_id(OchushaBBSThread *thread, const char *id)
   g_return_if_fail(OCHUSHA_IS_BBS_THREAD(thread));
 
   if (thread->id != NULL)
-    g_free(thread->id);
-  thread->id = (id != NULL ? g_strdup(id) : NULL);
+    G_FREE(thread->id);
+  thread->id = (id != NULL ? G_STRDUP(id) : NULL);
 
   g_object_notify(G_OBJECT(thread), "id");
 }
@@ -341,7 +347,7 @@ ochusha_bbs_thread_set_title(OchushaBBSThread *thread, const gchar *title)
   g_return_if_fail(OCHUSHA_IS_BBS_THREAD(thread));
 
   if (thread->title != NULL)
-    g_free(thread->title);
+    G_FREE(thread->title);
 
   thread->title = (title != NULL ? wipe_string(title) : NULL);
 
@@ -358,37 +364,29 @@ ochusha_bbs_thread_get_title(OchushaBBSThread *thread)
 }
 
 
-const char *
-ochusha_bbs_thread_get_response_character_encoding(OchushaBBSThread *thread)
-{
-  OchushaBBSThreadClass *klass;
-
-  g_return_val_if_fail(OCHUSHA_BBS_THREAD(thread), NULL);
-
-  klass = OCHUSHA_BBS_THREAD_GET_CLASS(thread);
-  g_return_val_if_fail(klass->get_response_character_encoding != NULL, NULL);
-
-  return (*klass->get_response_character_encoding)(thread);
-}
-
-
 OchushaAsyncBuffer *
 ochusha_bbs_thread_get_responses_source(OchushaBBSThread *thread,
-					OchushaConfig *conf,
-					const char *last_modified,
-					OchushaCacheMode mode,
-					OchushaNetworkCallbacks *callbacks,
-					gpointer callback_data)
+					OchushaNetworkBroker *broker,
+					OchushaNetworkBrokerCacheMode mode)
 {
   OchushaBBSThreadClass *klass;
 
-  g_return_val_if_fail(OCHUSHA_BBS_THREAD(thread) && conf != NULL, NULL);
+  g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread)
+		       && OCHUSHA_IS_NETWORK_BROKER(broker), NULL);
 
   klass = OCHUSHA_BBS_THREAD_GET_CLASS(thread);
   g_return_val_if_fail(klass->get_responses_source != NULL, NULL);
 
-  return (*klass->get_responses_source)(thread, conf, last_modified, mode,
-					callbacks, callback_data);
+  return (*klass->get_responses_source)(thread, broker, mode);
+}
+
+
+const char *
+ochusha_bbs_thread_get_response_character_encoding(OchushaBBSThread *thread)
+{
+  g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread)
+		       && OCHUSHA_IS_BULLETIN_BOARD(thread->board), NULL);
+  return ochusha_bulletin_board_get_response_character_encoding(OCHUSHA_BULLETIN_BOARD(thread->board));
 }
 
 
@@ -403,7 +401,7 @@ ochusha_bbs_thread_parse_responses(OchushaBBSThread *thread,
 {
   OchushaBBSThreadClass *klass;
 
-  g_return_val_if_fail(OCHUSHA_BBS_THREAD(thread) && buffer != NULL, FALSE);
+  g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread) && buffer != NULL, FALSE);
 
   klass = OCHUSHA_BBS_THREAD_GET_CLASS(thread);
   g_return_val_if_fail(klass->parse_responses != NULL, FALSE);
@@ -414,16 +412,95 @@ ochusha_bbs_thread_parse_responses(OchushaBBSThread *thread,
 }
 
 
+gboolean
+ochusha_bbs_thread_is_preview_supported(OchushaBBSThread *thread)
+{
+  OchushaBBSThreadClass *klass;
+
+  g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread), FALSE);
+
+  klass = OCHUSHA_BBS_THREAD_GET_CLASS(thread);
+  return klass->preview_response != NULL;
+}
+
+
+gboolean
+ochusha_bbs_thread_preview_response(OchushaBBSThread *thread,
+				    const OchushaBBSResponse *response,
+				    StartThreadCallback *start_cb,
+				    EachResponseCallback *response_cb,
+				    EndThreadCallback *end_cb,
+				    gpointer callback_data)
+{
+  OchushaBBSThreadClass *klass;
+
+  g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread) && response != NULL,
+		       FALSE);
+  klass = OCHUSHA_BBS_THREAD_GET_CLASS(thread);
+
+  g_return_val_if_fail(klass->preview_response != NULL, FALSE);
+
+  return (*klass->preview_response)(thread, response,
+				    start_cb, response_cb, end_cb,
+				    callback_data);
+}
+
+
+gboolean
+ochusha_bbs_thread_is_post_supported(OchushaBBSThread *thread)
+{
+  OchushaBBSThreadClass *klass;
+
+  g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread), FALSE);
+
+  klass = OCHUSHA_BBS_THREAD_GET_CLASS(thread);
+  return klass->post_response != NULL;
+}
+
+
+gboolean
+ochusha_bbs_thread_post_response(OchushaBBSThread *thread,
+				 OchushaConfig *config,
+				 const OchushaBBSResponse *response)
+{
+  OchushaBBSThreadClass *klass;
+
+  g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread)
+		       && config != NULL && response != NULL, FALSE);
+
+  klass = OCHUSHA_BBS_THREAD_GET_CLASS(thread);
+
+  g_return_val_if_fail(klass->post_response != NULL, FALSE);
+
+  return (*klass->post_response)(thread, config, response);
+}
+
+
 const char *
-ochusha_bbs_thread_get_thread_url(OchushaBBSThread *thread)
+ochusha_bbs_thread_get_url_to_post_response(OchushaBBSThread *thread)
 {
   OchushaBBSThreadClass *klass;
   g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread), NULL);
 
   klass = OCHUSHA_BBS_THREAD_GET_CLASS(thread);
-  g_return_val_if_fail(klass->get_thread_url != NULL, NULL);
+  if (klass->get_url_to_post != NULL)
+    return (*klass->get_url_to_post)(thread);
 
-  return (*klass->get_thread_url)(thread);
+  g_return_val_if_fail(klass->get_url != NULL, NULL);
+  return (*klass->get_url)(thread);
+}
+
+
+const char *
+ochusha_bbs_thread_get_url(OchushaBBSThread *thread)
+{
+  OchushaBBSThreadClass *klass;
+  g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread), NULL);
+
+  klass = OCHUSHA_BBS_THREAD_GET_CLASS(thread);
+  g_return_val_if_fail(klass->get_url != NULL, NULL);
+
+  return (*klass->get_url)(thread);
 }
 
 
