@@ -23,13 +23,15 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.h,v 1.3 2003/05/15 19:55:44 fuyu Exp $
+ * $Id: ochusha_bbs_thread.h,v 1.4 2003/05/25 08:46:16 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BBS_THREAD_H_
 #define _OCHUSHA_BBS_THREAD_H_
 
 #include "ochusha_bulletin_board.h"
+#include "ochusha_network_broker.h"
+
 #include <glib-object.h>
 #include <glib.h>
 
@@ -88,14 +90,16 @@ struct _OchushaBBSThreadClass
   void (*write_threadlist_element)(OchushaBBSThread *thread,
 				   FILE *threadlist_xml);
 
+
+  /* postξܤˤ륷ʥ */
+  gboolean (*check_cookie)(OchushaBBSThread *thread);
+  gboolean (*too_match_clouded)(OchushaBBSThread *thread);
+
+
   /* virtual function table */
-  const char *(*get_response_character_encoding)(OchushaBBSThread *thread);
   OchushaAsyncBuffer *(*get_responses_source)(OchushaBBSThread *thread,
-					      OchushaConfig *conf,
-					      const char *last_modified,
-					      OchushaCacheMode mode,
-					      OchushaNetworkCallbacks *cb,
-					      gpointer callback_data);
+					OchushaNetworkBroker *broker,
+					OchushaNetworkBrokerCacheMode mode);
   gboolean (*parse_responses)(OchushaBBSThread *thread,
 			      OchushaAsyncBuffer *buffer,
 			      int start, int number,
@@ -103,9 +107,19 @@ struct _OchushaBBSThreadClass
 			      EachResponseCallback *each_response_callback,
 			      EndThreadCallback *end_thread_callback,
 			      gpointer callback_data);
-  const char *(*get_thread_url)(OchushaBBSThread *thread);
+  const char *(*get_url)(OchushaBBSThread *thread);
+  const char *(*get_url_to_post)(OchushaBBSThread *thread);
   gboolean (*check_url)(OchushaBBSThread *thread, const char *url,
 			unsigned int *from_p, unsigned int *to_p);
+
+  gboolean (*preview_response)(OchushaBBSThread *thread,
+			       const OchushaBBSResponse *response,
+			       StartThreadCallback *start_cb,
+			       EachResponseCallback *response_cb,
+			       EndThreadCallback *end_cb,
+			       gpointer callback_data);
+  gboolean (*post_response)(OchushaBBSThread *thread, OchushaConfig *config,
+			    const OchushaBBSResponse *response);
 };
 
 
@@ -143,21 +157,23 @@ const gchar *ochusha_bbs_thread_get_title(OchushaBBSThread *thread);
 int ochusha_bbs_thread_get_number_of_responses_on_server(OchushaBBSThread *thread);
 int ochusha_bbs_thread_get_number_of_responses_read(OchushaBBSThread *thread);
 
-const char *ochusha_bbs_thread_get_thread_url(OchushaBBSThread *thread);
+const char *ochusha_bbs_thread_get_url(OchushaBBSThread *thread);
+
+gboolean ochusha_bbs_thread_is_preview_supported(OchushaBBSThread *thread);
+gboolean ochusha_bbs_thread_is_post_supported(OchushaBBSThread *thread);
+const char *ochusha_bbs_thread_get_url_to_post_response(
+						OchushaBBSThread *thread);
 
 gboolean ochusha_bbs_thread_check_url(OchushaBBSThread *thread,
 				      const char *url, unsigned int *from_p,
 				      unsigned int *end_p);
 
+OchushaAsyncBuffer *ochusha_bbs_thread_get_responses_source(
+					OchushaBBSThread *thread,
+					OchushaNetworkBroker *broker,
+					OchushaNetworkBrokerCacheMode mode);
 const char *ochusha_bbs_thread_get_response_character_encoding(
 						OchushaBBSThread *thread);
-OchushaAsyncBuffer *ochusha_bbs_thread_get_responses_source(
-						OchushaBBSThread *thread,
-						OchushaConfig *conf,
-						const char *last_modified,
-						OchushaCacheMode mode,
-						OchushaNetworkCallbacks *cb,
-						gpointer callback_data);
 gboolean ochusha_bbs_thread_parse_responses(OchushaBBSThread *thread,
 					    OchushaAsyncBuffer *buffer,
 					    int start, int number,
@@ -166,6 +182,17 @@ gboolean ochusha_bbs_thread_parse_responses(OchushaBBSThread *thread,
 					    EndThreadCallback *end_cb,
 					    gpointer callback_data);
 
+gboolean ochusha_bbs_thread_preview_response(OchushaBBSThread *thread,
+					const OchushaBBSResponse *response,
+					StartThreadCallback *start_cb,
+					EachResponseCallback *response_cb,
+					EndThreadCallback *end_cb,
+					gpointer callback_data);
+
+gboolean ochusha_bbs_thread_post_response(OchushaBBSThread *thread,
+					  OchushaConfig *config,
+					  const OchushaBBSResponse *response);
+
 #ifdef __cplusplus
 } // extern "C"
 #endif
