@@ -23,14 +23,17 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_category.c,v 1.2 2003/05/14 17:14:19 fuyu Exp $
+ * $Id: ochusha_board_category.c,v 1.4 2003/06/06 19:38:04 fuyu Exp $
  */
 
+#include "config.h"
 
 #include "ochusha_private.h"
 #include "ochusha_bulletin_board.h"
 #include "ochusha_board_category.h"
 
+#include "utils.h"
+
 #include "marshal.h"
 
 #include <glib-object.h>
@@ -168,7 +171,7 @@ ochusha_board_category_finalize(GObject *object)
 
   if (category->name != NULL)
     {
-      g_free(category->name);
+      G_FREE(category->name);
       category->name = NULL;
     }
 
@@ -240,7 +243,7 @@ ochusha_board_category_set_name(OchushaBoardCategory *category,
   g_return_if_fail(OCHUSHA_IS_BOARD_CATEGORY(category) && name != NULL);
 
   if (category->name != NULL)
-    g_free(category->name);
+    G_FREE(category->name);
 
   category->name = wipe_string(name);
 }
