@@ -23,9 +23,10 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.c,v 1.3 2003/05/14 20:58:12 fuyu Exp $
+ * $Id: ochusha_bulletin_board.c,v 1.6 2003/06/07 14:35:32 fuyu Exp $
  */
 
+#include "config.h"
 
 #include "ochusha_private.h"
 #include "ochusha_bulletin_board.h"
@@ -55,6 +56,9 @@ static void ochusha_bulletin_board_set_property(GObject *object, guint prop_id,
 static void ochusha_bulletin_board_get_property(GObject *object, guint prop_id,
 						GValue *value,
 						GParamSpec *pspec);
+static char *ochusha_bulletin_board_generate_board_id(
+						OchushaBulletinBoard *board,
+						const char *url);
 
 static void ochusha_bulletin_board_read_boardlist_element(
 						OchushaBulletinBoard *board,
@@ -189,6 +193,7 @@ ochusha_bulletin_board_class_init(OchushaBulletinBoardClass *klass)
   klass->threadlist_read_thread_element = NULL;
   klass->threadlist_write_thread_element = NULL;
 
+  klass->generate_board_id = ochusha_bulletin_board_generate_board_id;
   klass->thread_new = NULL;
   klass->lookup_thread_by_url = NULL;
   klass->get_threadlist_source = NULL;
@@ -203,6 +208,7 @@ ochusha_bulletin_board_init(OchushaBulletinBoard *board)
   board->base_url = NULL;
   board->server = NULL;
   board->base_path = NULL;
+  board->id = NULL;
 
   board->thread_list = NULL;
   board->thread_table = g_hash_table_new_full(g_str_hash, g_str_equal, NULL,
@@ -220,28 +226,34 @@ ochusha_bulletin_board_finalize(GObject *object)
 
   if (board->name != NULL)
     {
-      g_free(board->name);
+      G_FREE(board->name);
       board->name = NULL;
     }
 
   if (board->base_url != NULL)
     {
-      g_free(board->base_url);
+      G_FREE(board->base_url);
       board->base_url = NULL;
     }
 
   if (board->server != NULL)
     {
-      g_free(board->server);
+      G_FREE(board->server);
       board->server = NULL;
     }
 
   if (board->base_path != NULL)
     {
-      g_free(board->base_path);
+      G_FREE(board->base_path);
       board->base_path = NULL;
     }
 
+  if (board->id != NULL)
+    {
+      G_FREE(board->id);
+      board->id = NULL;
+    }
+
   if (board->thread_list != NULL)
     {
       g_slist_foreach(board->thread_list, (GFunc)g_object_unref, NULL);
@@ -351,6 +363,7 @@ void
 ochusha_bulletin_board_set_base_url(OchushaBulletinBoard *board,
 				    const char *url)
 {
+  OchushaBulletinBoardClass *klass;
   char *server;
   char *base_path;
 
@@ -363,16 +376,31 @@ ochusha_bulletin_board_set_base_url(OchushaBulletinBoard *board,
   g_return_if_fail(base_path != NULL);
 
   if (board->base_url != NULL)
-    g_free(board->base_url);
-  board->base_url = g_strdup(url);
+    G_FREE(board->base_url);
+  board->base_url = G_STRDUP(url);
 
   if (board->server != NULL)
-    g_free(board->server);
+    G_FREE(board->server);
   board->server = server;
 
   if (board->base_path != NULL)
-    g_free(board->base_path);
+    G_FREE(board->base_path);
   board->base_path = base_path;
+
+  if (board->id != NULL)
+    G_FREE(board->id);
+
+  klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
+  board->id = (*klass->generate_board_id)(board, url);
+}
+
+
+static char *
+ochusha_bulletin_board_generate_board_id(OchushaBulletinBoard *board,
+					 const char *url)
+{
+  g_warning("A sub class of OchushaBulletinBoard didn't implement generate_board_id()\n");
+  return NULL;
 }
 
 
@@ -403,6 +431,15 @@ ochusha_bulletin_board_get_base_path(OchushaBulletinBoard *board)
 }
 
 
+const char *
+ochusha_bulletin_board_get_id(OchushaBulletinBoard *board)
+{
+  g_return_val_if_fail(OCHUSHA_IS_BULLETIN_BOARD(board), NULL);
+
+  return board->id;
+}
+
+
 OchushaBBSThread *
 ochusha_bulletin_board_bbs_thread_new(OchushaBulletinBoard *board,
 				      const char *id, const gchar *title)
@@ -420,7 +457,12 @@ ochusha_bulletin_board_bbs_thread_new(OchushaBulletinBoard *board,
   klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
   g_return_val_if_fail(klass->thread_new != NULL, NULL);
 
-  return (*klass->thread_new)(board, id, title);
+  thread = (*klass->thread_new)(board, id, title);
+
+  if (thread != NULL)
+    g_hash_table_insert(board->thread_table, thread->id, thread);
+    
+  return thread;
 }
 
 
@@ -453,20 +495,17 @@ ochusha_bulletin_board_lookup_bbs_thread_by_url(OchushaBulletinBoard *board,
 
 OchushaAsyncBuffer *
 ochusha_bulletin_board_get_threadlist_source(OchushaBulletinBoard *board,
-					     OchushaConfig *conf,
-					     const char *last_modified,
-					     OchushaCacheMode mode,
-					     OchushaNetworkCallbacks *cb,
-					     gpointer callback_data)
+					OchushaNetworkBroker *broker,
+					OchushaNetworkBrokerCacheMode mode)
 {
   OchushaBulletinBoardClass *klass;
-  g_return_val_if_fail(OCHUSHA_IS_BULLETIN_BOARD(board) && conf != NULL, NULL);
+  g_return_val_if_fail(OCHUSHA_IS_BULLETIN_BOARD(board)
+		       && OCHUSHA_IS_NETWORK_BROKER(broker), NULL);
 
   klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
   g_return_val_if_fail(klass->get_threadlist_source != NULL, NULL);
 
-  return (*klass->get_threadlist_source)(board, conf, last_modified,
-					 mode, cb, callback_data);
+  return (*klass->get_threadlist_source)(board, broker, mode);
 }
 
 
@@ -487,6 +526,21 @@ ochusha_bulletin_board_refresh_threadlist(OchushaBulletinBoard *board,
 }
 
 
+const char *
+ochusha_bulletin_board_get_response_character_encoding(
+						OchushaBulletinBoard *board)
+{
+  OchushaBulletinBoardClass *klass;
+
+  g_return_val_if_fail(OCHUSHA_BULLETIN_BOARD(board), NULL);
+
+  klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
+  g_return_val_if_fail(klass->get_response_character_encoding != NULL, NULL);
+
+  return (*klass->get_response_character_encoding)(board);
+}
+
+
 /*
  * boardlist.xmlϢ
  */
@@ -564,13 +618,13 @@ cleanup_sax_context(SAXContext *context)
 {
   if (context->current_attr_name != NULL)
     {
-      free(context->current_attr_name);
+      G_FREE(context->current_attr_name);
       context->current_attr_name = NULL;
     }
 
   if (context->current_attr_val != NULL)
     {
-      free(context->current_attr_val);
+      G_FREE(context->current_attr_val);
       context->current_attr_val = NULL;
     }
 
@@ -582,6 +636,18 @@ cleanup_sax_context(SAXContext *context)
 }
 
 
+#if TRACE_MEMORY_USAGE
+static void
+trace_free(gpointer pointer)
+{
+  G_FREE(pointer);
+}
+#define TRACE_FREE	trace_free
+#else
+#define TRACE_FREE	g_free
+#endif
+
+
 static void
 startElementHandler(void *context, const xmlChar *name, const xmlChar **atts)
 {
@@ -598,7 +664,8 @@ startElementHandler(void *context, const xmlChar *name, const xmlChar **atts)
       if (strcmp(name, "threadlist") == 0)
 	{
 	  sax_context->thread_attributes
-	    = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	    = g_hash_table_new_full(g_str_hash, g_str_equal,
+				    TRACE_FREE, TRACE_FREE);
 	  sax_context->state = SAX_THREADLIST;
 	  return;
 	}
@@ -616,14 +683,14 @@ startElementHandler(void *context, const xmlChar *name, const xmlChar **atts)
 		      sax_context->current_attr_name,
 		      sax_context->current_attr_val);
 #endif
-	      g_free(sax_context->current_attr_name);
-	      g_free(sax_context->current_attr_val);
+	      G_FREE(sax_context->current_attr_name);
+	      G_FREE(sax_context->current_attr_val);
 	      sax_context->current_attr_name = NULL;
 	      sax_context->current_attr_val = NULL;
 	      break;	/* 顼 */
 	    }
 
-	  sax_context->current_attr_name = g_strdup(atts[1]);
+	  sax_context->current_attr_name = G_STRDUP(atts[1]);
 	  return;
 	}
       break;	/* 顼 */
@@ -654,10 +721,10 @@ startElementHandler(void *context, const xmlChar *name, const xmlChar **atts)
 		      sax_context->current_attr_name,
 		      sax_context->current_attr_val, atts[1]);
 #endif
-	      g_free(sax_context->current_attr_val);
+	      G_FREE(sax_context->current_attr_val);
 	    }
 
-	  sax_context->current_attr_val = g_strdup(atts[1]);
+	  sax_context->current_attr_val = G_STRDUP(atts[1]);
 	  return;
 	}
       else if (strcmp(name, "string") == 0)
@@ -743,7 +810,7 @@ endElementHandler(void *context, const xmlChar *name)
 	   * Ϣ뤹뤿ΰХåեɬפǤ뤫顣
 	   */
 	  sax_context->current_attr_val
-	    = g_strdup(sax_context->current_attr_val);
+	    = G_STRDUP(sax_context->current_attr_val);
 	  return;
 	}
       break;	/* 顼 */
@@ -779,22 +846,23 @@ endElementHandler(void *context, const xmlChar *name)
 
 	      if (thread == NULL)
 		{
-		  gchar *title
-		    = ochusha_utils_get_attribute_string(thread_attrs,
-							 "title");
+		  const gchar *title = g_hash_table_lookup(thread_attrs,
+							   "title");
 		  if (title != NULL)
-		    thread
-		      = ochusha_bulletin_board_bbs_thread_new(board,
-							      thread_id,
-							      title);
-		  g_signal_emit_by_name(G_OBJECT(thread),
-					"read_threadlist_element",
-					thread_attrs);
-		  g_signal_emit(G_OBJECT(board),
-				bulletin_board_signals[THREADLIST_READ_THREAD_ELEMENT_SIGNAL],
-				0,
-				thread,
-				thread_attrs);
+		    {
+		      thread
+			= ochusha_bulletin_board_bbs_thread_new(board,
+								thread_id,
+								title);
+		      g_signal_emit_by_name(G_OBJECT(thread),
+					    "read_threadlist_element",
+					    thread_attrs);
+		      g_signal_emit(G_OBJECT(board),
+				    bulletin_board_signals[THREADLIST_READ_THREAD_ELEMENT_SIGNAL],
+				    0,
+				    thread,
+				    thread_attrs);
+		    }
 		}
 	    }
 	   /* ꡼󥢥å */
@@ -917,14 +985,14 @@ ochusha_bulletin_board_read_threadlist_xml(OchushaBulletinBoard *board,
 
   if (context.state == SAX_ACCEPTED)
     {
-      g_free(threadlist_xml);
+      G_FREE(threadlist_xml);
       return TRUE;
     }
 
   fprintf(stderr, "%s is unacceptable as ochusha's threadlist.\n",
 	  threadlist_xml);
 
-  g_free(threadlist_xml);
+  G_FREE(threadlist_xml);
 
   return FALSE;
 }
