@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.h,v 1.2 2003/05/14 20:58:12 fuyu Exp $
+ * $Id: ochusha_bulletin_board.h,v 1.3 2003/05/25 08:46:16 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BULLETIN_BOARD_H_
@@ -31,6 +31,7 @@
 
 #include "ochusha.h"
 #include "ochusha_async_buffer.h"
+#include "ochusha_network_broker.h"
 
 #include <glib-object.h>
 #include <glib.h>
@@ -72,10 +73,10 @@ struct _OchushaBulletinBoard
   gchar *name;			/* 㤨 "UNIX" */
   char *base_url;		/* 㤨 "http://pc.2ch.net/unix/" */
 
-  /* ʲĤbase_url롣*/
+  /* ʲλĤbase_url롣*/
   char *server;			/* 㤨 "pc.2ch.net" */
   char *base_path;		/* 㤨 "/unix/" */
-
+  char *id;			/* 㤨 "unix" */
 
   GSList *thread_list;		/* Ƥ륹(OchushaBBSThread *)ꥹ */
   GHashTable *thread_table;	/* ID(DATե̾)=>(OchushaBBSThread *) */
@@ -102,20 +103,20 @@ struct _OchushaBulletinBoardClass
 					  FILE *threadlist_xml);
 
   /* virtual function table */
+  char *(*generate_board_id)(OchushaBulletinBoard *board, const char *url);
   OchushaBBSThread *(*thread_new)(OchushaBulletinBoard *board,
 				  const char *id, const gchar *title);
   OchushaBBSThread *(*lookup_thread_by_url)(OchushaBulletinBoard *board,
 					    const char *url);
   OchushaAsyncBuffer *(*get_threadlist_source)(OchushaBulletinBoard *board,
-					       OchushaConfig *config,
-					       const char *last_modified,
-					       OchushaCacheMode mode,
-					       OchushaNetworkCallbacks *cb,
-					       gpointer callback_data);
+					OchushaNetworkBroker *broker,
+					OchushaNetworkBrokerCacheMode mode);
   gboolean (*refresh_threadlist)(OchushaBulletinBoard *board,
 				 OchushaAsyncBuffer *buffer,
 				 EachThreadCallback *cb,
 				 gpointer callback_data);
+
+  const char *(*get_response_character_encoding)(OchushaBulletinBoard *board);
 };
 
 
@@ -133,6 +134,7 @@ const char *ochusha_bulletin_board_get_base_url(OchushaBulletinBoard *board);
 
 const char *ochusha_bulletin_board_get_server(OchushaBulletinBoard *board);
 const char *ochusha_bulletin_board_get_base_path(OchushaBulletinBoard *board);
+const char *ochusha_bulletin_board_get_id(OchushaBulletinBoard *board);
 
 OchushaBBSThread *ochusha_bulletin_board_bbs_thread_new(
 					OchushaBulletinBoard *board,
@@ -146,16 +148,16 @@ OchushaBBSThread *ochusha_bulletin_board_lookup_bbs_thread_by_url(
 
 OchushaAsyncBuffer *ochusha_bulletin_board_get_threadlist_source(
 					OchushaBulletinBoard *board,
-					OchushaConfig *config,
-					const char *last_modified,
-					OchushaCacheMode mode,
-					OchushaNetworkCallbacks *callbacks,
-					gpointer callback_data);
+					OchushaNetworkBroker *broker,
+					OchushaNetworkBrokerCacheMode mode);
 
 gboolean ochusha_bulletin_board_refresh_threadlist(OchushaBulletinBoard *board,
 						   OchushaAsyncBuffer *buffer,
 						   EachThreadCallback *cb,
 						   gpointer callback_data);
+const char *ochusha_bulletin_board_get_response_character_encoding(
+						OchushaBulletinBoard *board);
+
 
 gboolean ochusha_bulletin_board_read_threadlist_xml(
 					OchushaBulletinBoard *board,
