@@ -23,12 +23,16 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_config.c,v 1.2 2003/05/10 17:45:31 fuyu Exp $
+ * $Id: ochusha_config.c,v 1.6 2003/06/07 14:35:32 fuyu Exp $
  */
 
+#include "config.h"
+
 #include "ochusha_private.h"
 #include "ochusha.h"
 
+#include "utils.h"
+
 #include <glib.h>
 #include <libxml/SAX.h>
 #include <libxml/parser.h>
@@ -38,6 +42,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
 
 
@@ -65,10 +71,10 @@ ochusha_config_prepare_home(OchushaConfig *config)
       if (snprintf(buffer, PATH_MAX, "%s/.ochusha", home) >= PATH_MAX)
 	goto no_home_exit;
 
-      ochusha_home = g_strdup(buffer);
+      ochusha_home = G_STRDUP(buffer);
     }
   else
-    ochusha_home = g_strdup(ochusha_home);
+    ochusha_home = G_STRDUP(ochusha_home);
 
   if (mkdir_p(ochusha_home))
     {
@@ -78,7 +84,7 @@ ochusha_config_prepare_home(OchushaConfig *config)
 
   fprintf(stderr, "Couldn't create directory %s: %s\n", ochusha_home,
 	  strerror(errno));
-  g_free(ochusha_home);
+  G_FREE(ochusha_home);
 
  no_home_exit:
   config->home = NULL;
@@ -94,9 +100,10 @@ ochusha_config_prepare_home(OchushaConfig *config)
  * callerϥѥ̾ʸפˤʤäfree()̳餦
  */
 char *
-ochusha_config_find_file(OchushaConfig *config, const char *filename)
+ochusha_config_find_file(const OchushaConfig *config, const char *filename)
 {
-  int fd;
+  int stat_result;
+  struct stat sb;
   char pathname[PATH_MAX];
 
   if (config->home == NULL)
@@ -106,13 +113,9 @@ ochusha_config_find_file(OchushaConfig *config, const char *filename)
       >= PATH_MAX)
     return NULL;
 
-  /* Ƥߤ¸˽ */
-  fd = open(pathname, O_RDONLY);
-  if (fd >= 0)
-    {
-      close(fd);
-      return g_strdup(pathname);
-    }
+  stat_result = stat(pathname, &sb);
+  if (stat_result == 0 && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+    return G_STRDUP(pathname);
 
   return NULL;
 }
@@ -125,7 +128,7 @@ ochusha_config_find_file(OchushaConfig *config, const char *filename)
  * flagsͿ줿⡼ɤǳ
  */
 int
-ochusha_config_open_file(OchushaConfig *config, const char *filename,
+ochusha_config_open_file(const OchushaConfig *config, const char *filename,
 			 int flags)
 {
   int result;
@@ -164,6 +167,49 @@ ochusha_config_open_file(OchushaConfig *config, const char *filename,
 }
 
 
+/*
+ * ochusha_config_cache_file_exist:
+ *
+ * ꤵ줿URLФ륭åե뤬뤫ɤĴ٤롣
+ *
+ * å夬¸ߤȤˤTRUE֤
+ */
+gboolean
+ochusha_config_cache_file_exist(const OchushaConfig *config, const char *url)
+{
+  char *cur_pos;
+  char pathname[PATH_MAX];
+  int stat_result;
+  struct stat sb;
+
+  if (url == NULL)
+    return FALSE;
+
+  if (config->home == NULL)
+    return FALSE;
+
+  if (url[strlen(url) - 1] == '/')
+    return FALSE;
+
+  /*
+   * ݤʤΤurlhttp://server/.*ǤȷǤ
+   */
+  cur_pos = strstr(url, "http://");
+  if (cur_pos == NULL)
+    return FALSE;
+
+  if (snprintf(pathname, PATH_MAX, "%s/cache/%s", config->home, url + 7)
+      >= PATH_MAX)
+    return FALSE;
+
+  stat_result = stat(pathname, &sb);
+  if (stat_result == 0 && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+    return TRUE;
+
+  return FALSE;
+}
+
+
 /*
  * ochusha_config_cache_open_file:
  *
@@ -171,12 +217,9 @@ ochusha_config_open_file(OchushaConfig *config, const char *filename,
  * flagsopenƥॳͿflagsǤ롣
  */
 int
-ochusha_config_cache_open_file(OchushaConfig *config, const char *url,
+ochusha_config_cache_open_file(const OchushaConfig *config, const char *url,
 			       int flags)
 {
-  /*
-   * ݤʤΤurlhttp://server/.*ǤȷǤ
-   */
   char *cur_pos;
   char pathname[PATH_MAX];
   int len;
@@ -191,6 +234,10 @@ ochusha_config_cache_open_file(OchushaConfig *config, const char *url,
   if (url[strlen(url) - 1] == '/')
     return -1;
 
+  /*
+   * ݤʤΤurlhttp://server/.*ǤȷǤ
+   */
+
   cur_pos = strstr(url, "http://");
   if (cur_pos == NULL)
     return -1;
@@ -231,7 +278,7 @@ ochusha_config_cache_open_file(OchushaConfig *config, const char *url,
  * URLб륭åե뤬¸ߤunlink롣
  */
 void
-ochusha_config_cache_unlink_file(OchushaConfig *config, const char *url)
+ochusha_config_cache_unlink_file(const OchushaConfig *config, const char *url)
 {
   /*
    * ݤʤΤurlhttp://server/.*ǤȷǤ
@@ -267,7 +314,7 @@ ochusha_config_cache_unlink_file(OchushaConfig *config, const char *url)
 
   unlink(pathname);
 
-  g_free(pathname);
+  G_FREE(pathname);
 }
 
 
@@ -303,8 +350,15 @@ char *
 ochusha_utils_get_attribute_string(GHashTable *hash_table,
 				   const char *attr_name)
 {
+#if 0
   char *str = g_hash_table_lookup(hash_table, attr_name);
-  return str != NULL ? g_strdup(str) : NULL;
+  char *result = str != NULL ? G_STRDUP(str) : NULL;
+  fprintf(stderr, "result(%p)=\"%s\"\n", result, result);
+  return result;
+#else
+  char *str = g_hash_table_lookup(hash_table, attr_name);
+  return str != NULL ? G_STRDUP(str) : NULL;
+#endif
 }
 
 
@@ -338,13 +392,13 @@ cleanup_sax_context(SAXContext *context)
 {
   if (context->current_attr_name != NULL)
     {
-      free(context->current_attr_name);
+      G_FREE(context->current_attr_name);
       context->current_attr_name = NULL;
     }
 
   if (context->current_attr_val != NULL)
     {
-      free(context->current_attr_val);
+      G_FREE(context->current_attr_val);
       context->current_attr_val = NULL;
     }
 
@@ -356,6 +410,18 @@ cleanup_sax_context(SAXContext *context)
 }
 
 
+#if TRACE_MEMORY_USAGE
+static void
+trace_free(gpointer pointer)
+{
+  G_FREE(pointer);
+}
+#define TRACE_FREE	trace_free
+#else
+#define TRACE_FREE	g_free
+#endif
+
+
 static void
 startElementHandler(void *context, const xmlChar *name, const xmlChar **atts)
 {
@@ -372,7 +438,8 @@ startElementHandler(void *context, const xmlChar *name, const xmlChar **atts)
       if (strcmp(name, "preference") == 0)
 	{
 	  sax_context->pref_attributes
-	    = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	    = g_hash_table_new_full(g_str_hash, g_str_equal,
+				    TRACE_FREE, TRACE_FREE);
 	  sax_context->state = SAX_PREFERENCE;
 	  return;
 	}
@@ -391,13 +458,13 @@ startElementHandler(void *context, const xmlChar *name, const xmlChar **atts)
 		      sax_context->current_attr_name,
 		      sax_context->current_attr_val);
 #endif
-	      free(sax_context->current_attr_name);
-	      free(sax_context->current_attr_val);
+	      G_FREE(sax_context->current_attr_name);
+	      G_FREE(sax_context->current_attr_val);
 	      sax_context->current_attr_name = NULL;
 	      sax_context->current_attr_val = NULL;
 	      break;	/* 顼 */
 	    }
-	  sax_context->current_attr_name = strdup(atts[1]);
+	  sax_context->current_attr_name = G_STRDUP(atts[1]);
 	  return;
 	}
       break;	/* 顼 */
@@ -432,10 +499,10 @@ startElementHandler(void *context, const xmlChar *name, const xmlChar **atts)
 		      sax_context->current_attr_name,
 		      sax_context->current_attr_val, atts[1]);
 #endif
-	      free(sax_context->current_attr_val);
+	      G_FREE(sax_context->current_attr_val);
 	    }
 
-	  sax_context->current_attr_val = strdup(atts[1]);
+	  sax_context->current_attr_val = G_STRDUP(atts[1]);
 	  return;
 	}
       else if (strcmp(name, "string") == 0)
@@ -507,8 +574,9 @@ endElementHandler(void *context, const xmlChar *name)
 	   * ƤSAXѡŪʣǤȤƸƤޤ礬ꡢ
 	   * Ϣ뤹뤿ΰХåեɬפǤ뤫顣
 	   */
-	  sax_context->current_attr_val
-	    = g_strdup(sax_context->current_attr_val);
+	  if (sax_context->current_attr_val != NULL)
+	    sax_context->current_attr_val
+	      = G_STRDUP(sax_context->current_attr_val);
 	  return;
 	}
       break;	/* 顼 */
@@ -582,23 +650,23 @@ getEntityHandler(void *context, const xmlChar *name)
 /*
  * config.xmlϢ
  */
-#define OUTPUT_CONFIG_ATTRIBUTE_INT(file, conf, attribute)	\
+#define OUTPUT_CONFIG_ATTRIBUTE_INT(file, config, attribute)	\
   fprintf(file,							\
 	  "    <attribute name=\"" #attribute	"\">\n"		\
 	  "      <int val=\"%d\"/>\n"				\
-	  "    </attribute>\n", (conf)->attribute)
+	  "    </attribute>\n", (config)->attribute)
 
-#define OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, conf, attribute)	\
+#define OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, config, attribute)\
   fprintf(file,							\
 	  "    <attribute name=\"" #attribute	"\">\n"		\
 	  "      <boolean val=\"%s\"/>\n"			\
-	  "    </attribute>\n", (conf)->attribute ? "true" : "false" )
+	  "    </attribute>\n", (config)->attribute ? "true" : "false" )
 
-#define OUTPUT_CONFIG_ATTRIBUTE_STRING(file, conf, attribute)	\
+#define OUTPUT_CONFIG_ATTRIBUTE_STRING(file, config, attribute)	\
   fprintf(file,							\
 	  "    <attribute name=\"" #attribute	"\">\n"		\
 	  "      <string>%s</string>\n"				\
-	  "    </attribute>\n", (conf)->attribute)
+	  "    </attribute>\n", (config)->attribute)
 
 
 gboolean
@@ -609,6 +677,7 @@ ochusha_write_config_xml(OchushaConfig *config,
   FILE *config_xml;
   int len;
   char pathname[PATH_MAX];
+  char *empty_string = "";
 
   if (config->home == NULL)
     {
@@ -639,6 +708,28 @@ ochusha_write_config_xml(OchushaConfig *config,
   OUTPUT_CONFIG_ATTRIBUTE_STRING(config_xml, config, bbsmenu_url);
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(config_xml, config, offline);
 
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(config_xml, config, enable_proxy);
+
+  if (config->proxy_url == NULL)
+    config->proxy_url = empty_string;
+  OUTPUT_CONFIG_ATTRIBUTE_STRING(config_xml, config, proxy_url);
+  if (config->proxy_url == empty_string)
+    config->proxy_url = NULL;
+
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(config_xml, config, enable_proxy_auth);
+
+  if (config->proxy_user == NULL)
+    config->proxy_user = empty_string;
+  OUTPUT_CONFIG_ATTRIBUTE_STRING(config_xml, config, proxy_user);
+  if (config->proxy_user == empty_string)
+    config->proxy_user = NULL;
+
+  if (config->proxy_password == NULL)
+    config->proxy_password = empty_string;
+  OUTPUT_CONFIG_ATTRIBUTE_STRING(config_xml, config, proxy_password);
+  if (config->proxy_password == empty_string)
+    config->proxy_password = empty_string;
+
   if (optional != NULL)
     (*optional)(config_xml, user_data);
   else
@@ -665,8 +756,16 @@ ochusha_read_config_xml(OchushaConfig *config,
   char *pathname = ochusha_config_find_file(config, OCHUSHA_CONFIG_XML);
   if (pathname == NULL)
     {
-      config->bbsmenu_url = g_strdup(OCHUSHA_DEFAULT_BBSMENU_URL);
+      config->bbsmenu_url = G_STRDUP(OCHUSHA_DEFAULT_BBSMENU_URL);
       config->offline = FALSE;
+
+      config->enable_proxy = FALSE;
+      config->enable_proxy_auth = FALSE;
+
+      config->proxy_url = G_STRDUP("");
+      config->proxy_user = G_STRDUP("");
+      config->proxy_password = G_STRDUP("");
+
       return;
     }
 
@@ -689,12 +788,32 @@ ochusha_read_config_xml(OchushaConfig *config,
   if (context.state == SAX_ACCEPTED)
     {
       GHashTable *pref = context.pref_attributes;
-      context.pref_attributes = NULL;
 
-      config->bbsmenu_url = ochusha_utils_get_attribute_string(pref,
-							       "bbsmenu_url");
+      config->bbsmenu_url
+	= ochusha_utils_get_attribute_string(pref, "bbsmenu_url");
       config->offline = ochusha_utils_get_attribute_boolean(pref, "offline");
 
+      config->enable_proxy
+	= ochusha_utils_get_attribute_boolean(pref, "enable_proxy");
+
+      config->proxy_url
+	= ochusha_utils_get_attribute_string(pref, "proxy_url");
+      if (config->proxy_url == NULL)
+	config->proxy_url = G_STRDUP("");
+
+      config->enable_proxy_auth
+	= ochusha_utils_get_attribute_boolean(pref, "enable_proxy_auth");
+
+      config->proxy_user
+	= ochusha_utils_get_attribute_string(pref, "proxy_user");
+      if (config->proxy_user == NULL)
+	config->proxy_user = G_STRDUP("");
+
+      config->proxy_password
+	= ochusha_utils_get_attribute_string(pref, "proxy_password");
+      if (config->proxy_password == NULL)
+	config->proxy_password = G_STRDUP("");
+
       if (optional != NULL)
 	(*optional)(pref, user_data);
       else
@@ -704,7 +823,7 @@ ochusha_read_config_xml(OchushaConfig *config,
     fprintf(stderr, "%s is unacceptable as an ochusha's config.xml.\n",
 	    pathname);
 
-  free(pathname);
+  G_FREE(pathname);
 
   cleanup_sax_context(&context);
 
