@@ -23,11 +23,13 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.2 2003/05/10 23:27:52 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.6 2003/06/07 14:35:32 fuyu Exp $
  */
 
-#include "ochusha.h"
+#include "config.h"
+
 #include "ochusha_private.h"
+#include "ochusha.h"
 #include "ochusha_thread_2ch.h"
 
 #include <glib-object.h>
@@ -49,6 +51,13 @@ static void ochusha_thread_2ch_class_init(OchushaThread2chClass *klass);
 static void ochusha_thread_2ch_init(OchushaThread2ch *thread);
 static void ochusha_thread_2ch_finalize(GObject *object);
 
+static void ochusha_thread_2ch_read_threadlist_element(
+						OchushaBBSThread *thread,
+						GHashTable *thread_attributes);
+static void ochusha_thread_2ch_write_threadlist_element(
+						OchushaBBSThread *thread,
+						FILE *threadlist_xml);
+
 static void ochusha_thread_2ch_property_change_notify(OchushaThread2ch *thread);
 
 static gboolean ochusha_thread_2ch_check_url(OchushaBBSThread *thread,
@@ -56,15 +65,10 @@ static gboolean ochusha_thread_2ch_check_url(OchushaBBSThread *thread,
 					     unsigned int *from_p,
 					     unsigned int *to_p);
 
-static const char *ochusha_thread_2ch_get_response_character_encoding(
-						OchushaBBSThread *thread);
 static OchushaAsyncBuffer *ochusha_thread_2ch_get_responses_source(
-						OchushaBBSThread *thread,
-						OchushaConfig *conf,
-						const char *last_modified,
-						OchushaCacheMode mode,
-						OchushaNetworkCallbacks *cb,
-						gpointer callback_data);
+					OchushaBBSThread *thread,
+					OchushaNetworkBroker *broker,
+					OchushaNetworkBrokerCacheMode mode);
 static gboolean ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 						   OchushaAsyncBuffer *buffer,
 						   int start, int number,
@@ -73,6 +77,19 @@ static gboolean ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 						   EndThreadCallback *end_cb,
 						   gpointer callback_data);
 
+static gboolean ochusha_thread_2ch_preview_response(OchushaBBSThread *thread,
+					const OchushaBBSResponse *response,
+					StartThreadCallback *start_cb,
+					EachResponseCallback *response_cb,
+					EndThreadCallback *end_cb,
+					gpointer callback_data);
+
+static gboolean ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
+					OchushaConfig *config,
+					const OchushaBBSResponse *response);
+
+static const char *ochusha_thread_2ch_get_url_to_post(OchushaBBSThread *thread);
+
 
 GType
 ochusha_thread_2ch_get_type(void)
@@ -116,13 +133,24 @@ ochusha_thread_2ch_class_init(OchushaThread2chClass *klass)
 
   o_class->finalize = ochusha_thread_2ch_finalize;
 
-  b_class->get_response_character_encoding
-    = ochusha_thread_2ch_get_response_character_encoding;
+  b_class->read_threadlist_element
+    = ochusha_thread_2ch_read_threadlist_element;
+  b_class->write_threadlist_element
+    = ochusha_thread_2ch_write_threadlist_element;
   b_class->get_responses_source = ochusha_thread_2ch_get_responses_source;
   b_class->parse_responses = ochusha_thread_2ch_parse_responses;
-  b_class->get_thread_url
+  b_class->get_url
     = (const char *(*)(OchushaBBSThread *))ochusha_thread_2ch_get_base_url;
+  b_class->get_url_to_post = ochusha_thread_2ch_get_url_to_post;
   b_class->check_url = ochusha_thread_2ch_check_url;
+
+#if 0
+  b_class->preview_response = NULL;
+  b_class->post_response = NULL;
+#else
+  b_class->preview_response = ochusha_thread_2ch_preview_response;
+  b_class->post_response = ochusha_thread_2ch_post_response;
+#endif
 }
 
 
@@ -132,6 +160,7 @@ ochusha_thread_2ch_init(OchushaThread2ch *thread)
   thread->dat_url = NULL;
   thread->base_url = NULL;
   thread->base_path = NULL;
+  thread->post_url = NULL;
 
   thread->responses = NULL;
 
@@ -155,40 +184,86 @@ ochusha_thread_2ch_finalize(GObject *object)
 
   if (thread->dat_url != NULL)
     {
-      g_free(thread->dat_url);
+      G_FREE(thread->dat_url);
       thread->dat_url = NULL;
     }
 
   if (thread->base_url != NULL)
     {
-      g_free(thread->base_url);
+      G_FREE(thread->base_url);
       thread->base_url = NULL;
     }
 
   if (thread->base_path != NULL)
     {
-      g_free(thread->base_path);
+      G_FREE(thread->base_path);
       thread->base_path = NULL;
     }
 
   if (thread->responses != NULL)
     {
-      g_free(thread->responses);
+      G_FREE(thread->responses);
       thread->responses = NULL;
     }
 
+  if (thread->last_modified != NULL)
+    {
+      G_FREE(thread->last_modified);
+      thread->last_modified = NULL;
+    }
+
   if (G_OBJECT_CLASS(parent_class)->finalize)
     (*G_OBJECT_CLASS(parent_class)->finalize)(object);
 }
 
 
+static void
+ochusha_thread_2ch_read_threadlist_element(OchushaBBSThread *thread,
+					   GHashTable *thread_attributes)
+{
+  OchushaThread2ch *thread_2ch = OCHUSHA_THREAD_2CH(thread);
+  thread_2ch->last_modified
+    = ochusha_utils_get_attribute_string(thread_attributes, "last_modified");
+
+  if (parent_class->read_threadlist_element != NULL)
+    (*parent_class->read_threadlist_element)(thread, thread_attributes);
+}
+
+
+#define OUTPUT_THREAD_ATTRIBUTE_STRING(file, thread, attribute)		\
+  do {									\
+    if ((thread)->attribute != NULL)					\
+      {									\
+        gchar *text = g_markup_escape_text((thread)->attribute, -1);	\
+        fprintf(file,							\
+	  "      <attribute name=\"" #attribute	"\">\n"			\
+	  "        <string>%s</string>\n"				\
+	  "      </attribute>\n", text);				\
+        g_free(text);							\
+      }									\
+  } while (0)
+
+
+static void
+ochusha_thread_2ch_write_threadlist_element(OchushaBBSThread *thread,
+					    FILE *threadlist_xml)
+{
+  OchushaThread2ch *thread_2ch = OCHUSHA_THREAD_2CH(thread);
+
+  if (parent_class->write_threadlist_element != NULL)
+    (*parent_class->write_threadlist_element)(thread, threadlist_xml);
+
+  OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, thread_2ch, last_modified);
+}
+
+
 static void
 ochusha_thread_2ch_property_change_notify(OchushaThread2ch *thread)
 {
   OchushaThread2ch *thread_2ch;
 #if DEBUG_LIB_OCHUSHA
-  fprintf(stderr, "thread(0x%x)->board changed to board(0x%x).\n",
-	  (int)thread, (int)thread->board);
+  fprintf(stderr, "thread(%p)->board changed to board(%p).\n",
+	  thread, thread->board);
 #endif
   g_return_if_fail(OCHUSHA_IS_THREAD_2CH(thread));
 
@@ -196,21 +271,27 @@ ochusha_thread_2ch_property_change_notify(OchushaThread2ch *thread)
 
   if (thread->dat_url != NULL)
     {
-      g_free(thread->dat_url);
+      G_FREE(thread->dat_url);
       thread->dat_url = NULL;
     }
 
   if (thread->base_url != NULL)
     {
-      g_free(thread->base_url);
+      G_FREE(thread->base_url);
       thread->base_url = NULL;
     }
 
   if (thread->base_path != NULL)
     {
-      g_free(thread->base_path);
+      G_FREE(thread->base_path);
       thread->base_path = NULL;
     }
+
+  if (thread->post_url != NULL)
+    {
+      G_FREE(thread->post_url);
+      thread->post_url = NULL;
+    }
 }
 
 
@@ -291,7 +372,7 @@ ochusha_thread_2ch_get_dat_url(OchushaThread2ch *thread_2ch)
 	       thread->board->base_url, thread->id) >= PATH_MAX)
     return NULL;
 
-  thread_2ch->dat_url = g_strdup(url);
+  thread_2ch->dat_url = G_STRDUP(url);
   return thread_2ch->dat_url;
 }
 
@@ -319,7 +400,7 @@ ochusha_thread_2ch_get_base_url(OchushaThread2ch *thread_2ch)
 	       thread->board->server, base_path) >= PATH_MAX)
     return NULL;
 
-  thread_2ch->base_url = g_strdup(url);
+  thread_2ch->base_url = G_STRDUP(url);
   return thread_2ch->base_url;
 }
 
@@ -351,43 +432,25 @@ ochusha_thread_2ch_get_base_path(OchushaThread2ch *thread_2ch)
       tmp_pos[1] = '\0';
     }
 
-  thread_2ch->base_path = g_strdup(base_path);
+  thread_2ch->base_path = G_STRDUP(base_path);
   return thread_2ch->base_path;
 }
 
 
-static const char *
-ochusha_thread_2ch_get_response_character_encoding(OchushaBBSThread *thread)
-{
-  g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread), NULL);
-
-  switch (thread->board->bbs_type)
-    {
-    case OCHUSHA_BBS_TYPE_2CH:
-      return "CP932";
-
-    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-      return "EUC-JP";
-
-    default:
-      return NULL;
-    }
-}
-
-
 static OchushaAsyncBuffer *
 ochusha_thread_2ch_get_responses_source(OchushaBBSThread *thread,
-					OchushaConfig *conf,
-					const char *last_modified,
-					OchushaCacheMode mode,
-					OchushaNetworkCallbacks *callbacks,
-					gpointer callback_data)
+					OchushaNetworkBroker *broker,
+					OchushaNetworkBrokerCacheMode mode)
 {
-  g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread) && conf != NULL, NULL);
+  OchushaThread2ch *thread_2ch;
+  g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread)
+		       && OCHUSHA_IS_NETWORK_BROKER(broker), NULL);
 
-  return ochusha_read_from_url(conf,
-		ochusha_thread_2ch_get_dat_url(OCHUSHA_THREAD_2CH(thread)),
-		last_modified, mode, callbacks, callback_data);
+  thread_2ch = OCHUSHA_THREAD_2CH(thread);
+
+  return ochusha_network_broker_read_from_url(broker,
+				ochusha_thread_2ch_get_dat_url(thread_2ch),
+				thread_2ch->last_modified, mode);
 }
 
 
@@ -423,7 +486,7 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
     {
       int rest;
       /* 줿DATǡ */
-      clone_buffer = g_malloc(eol_pos - cur_pos);
+      clone_buffer = G_MALLOC(eol_pos - cur_pos);
       memcpy(clone_buffer, start_pos, eol_pos - cur_pos);
       /*
        * XXX: ԡƤ'\0'ŬʸѤΤȡ
@@ -446,9 +509,9 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
 
 #if DEBUG_DAT
       {
-	char *broken_line = g_strndup(cur_pos, eol_pos - cur_pos);
+	char *broken_line = G_STRNDUP(cur_pos, eol_pos - cur_pos);
 	fprintf(stderr, "broken line: %s\n", broken_line);
-	g_free(broken_line);
+	G_FREE(broken_line);
       }
 #endif
     }
@@ -483,7 +546,7 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
 
   tmp_pos = comma_delimiter ? memchr(cur_pos, ',', eol_pos - cur_pos)
                             : g_strstr_len(cur_pos, eol_pos - cur_pos, "<>");
-  response->name = g_strndup(cur_pos, tmp_pos - cur_pos);
+  response->name = G_STRNDUP(cur_pos, tmp_pos - cur_pos);
 
   cur_pos = tmp_pos + (comma_delimiter ? 1 : 2);
   /* DATǡƤ뤳Ȥ⤢ */
@@ -493,7 +556,7 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
   tmp_pos = comma_delimiter ? memchr(cur_pos, ',', eol_pos - cur_pos)
                             : g_strstr_len(cur_pos, eol_pos - cur_pos, "<>");
   if (tmp_pos - cur_pos > 0)
-    response->mailto = g_strndup(cur_pos, tmp_pos - cur_pos);
+    response->mailto = G_STRNDUP(cur_pos, tmp_pos - cur_pos);
   else
     response->mailto = NULL;
 
@@ -507,7 +570,7 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
   if (tmp_pos == NULL)
     goto error_exit;
 
-  response->date_id = g_strndup(cur_pos, tmp_pos - cur_pos);
+  response->date_id = G_STRNDUP(cur_pos, tmp_pos - cur_pos);
 
   cur_pos = tmp_pos + (comma_delimiter ? 1 : 2);
   /* DATǡƤ뤳Ȥ⤢ */
@@ -518,11 +581,11 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
   if (tmp_pos == NULL)
     goto error_exit;
 
-  response->content = g_strndup(cur_pos, tmp_pos - cur_pos);
+  response->content = G_STRNDUP(cur_pos, tmp_pos - cur_pos);
 
   if (clone_buffer != NULL)
     {
-      g_free(clone_buffer);
+      G_FREE(clone_buffer);
       return start_pos + (tmp_pos - clone_buffer) + 2;
     }
   else
@@ -530,16 +593,16 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
 
  error_exit:
 #if DEBUG_DAT_MOST
-  tmp_pos = g_strndup(start_pos, eol_pos - start_pos);
+  tmp_pos = G_STRNDUP(start_pos, eol_pos - start_pos);
   fprintf(stderr, "DAT Parsing failed: \"%s\"", tmp_pos);
-  g_free(tmp_pos);
+  G_FREE(tmp_pos);
 #endif
   if (response->name != NULL)
     {
 #if DEBUG_DAT
       fprintf(stderr, "name=%s\n", response->name);
 #endif
-      g_free((gpointer)response->name);
+      G_FREE((gpointer)response->name);
       response->name = NULL;
     }
 
@@ -548,7 +611,7 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
 #if DEBUG_DAT
       fprintf(stderr, "mailto=%s\n", response->mailto);
 #endif
-      g_free((gpointer)response->mailto);
+      G_FREE((gpointer)response->mailto);
       response->mailto = NULL;
     }
 
@@ -557,7 +620,7 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
 #if DEBUG_DAT
       fprintf(stderr, "date_id=%s\n", response->date_id);
 #endif
-      g_free((gpointer)response->date_id);
+      G_FREE((gpointer)response->date_id);
       response->date_id = NULL;
     }
 
@@ -566,12 +629,12 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
 #if DEBUG_DAT
       fprintf(stderr, "content=%s\n", response->content);
 #endif
-      g_free((gpointer)response->content);
+      G_FREE((gpointer)response->content);
       response->content = NULL;
     }
 
   if (clone_buffer != NULL)
-    g_free(clone_buffer);
+    G_FREE(clone_buffer);
 
   return NULL;
 }
@@ -610,8 +673,15 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
   OchushaThread2ch *thread_2ch;
   unsigned int *responses;
   gboolean result = TRUE;
+  OchushaNetworkBrokerBufferStatus *status;
 
-  g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread) && buffer != NULL, FALSE);
+  g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread)
+		       && OCHUSHA_IS_ASYNC_BUFFER(buffer), FALSE);
+
+  status = g_object_get_data(G_OBJECT(buffer),
+			     "OchushaNetworkBroker::BufferStatus");
+
+  g_return_val_if_fail(status != NULL, FALSE);
 
   if (!ochusha_async_buffer_active_ref(buffer, "ochusha_thread_2ch.c: ochusha_thread_2ch_parse_responses"))
     {
@@ -622,6 +692,7 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
     }
 
   thread_2ch = OCHUSHA_THREAD_2CH(thread);
+
   responses = thread_2ch->responses;
 
   if (responses == NULL)
@@ -629,7 +700,7 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
       /*
        * 1000쥹ʬǤ⡹4000ХȤʤΤǸĹ϶Ȥޤ衣
        */
-      responses = g_new0(unsigned int, MAX_RESPONSE);
+      responses = G_NEW0(unsigned int, MAX_RESPONSE);
       thread_2ch->responses = responses;
     }
 
@@ -647,8 +718,7 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
     rest_of_responses = number;
     offset = 0;
     i = 0;
-    if (((OchushaNetworkStatus *)buffer->user_data)->state
-	!= OCHUSHA_CACHE_IS_DIRTY)
+    if (status->state != OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY)
       {
 	if (start > 1 && start < MAX_RESPONSE)
 	  {
@@ -672,19 +742,19 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	  }
       }
 
-    while (TRUE)
+    while (result)
       {
 	char *buffer_top = (char *)buffer->buffer;
 	char *cur_pos = buffer_top + offset;
-	int length = buffer->length;
-	int rest_of_data = length - offset;
-	char *eol_pos;
+	int rest_of_data = buffer->length - offset;
+	char *eol_pos = NULL;
 	int interval = ANALYSIS_INTERVAL;
 
 #if DEBUG_DAT_MOST
-	fprintf(stderr, "1: ochusha_thread_2ch.c: length=%d\n", length);
+	fprintf(stderr, "1: ochusha_thread_2ch.c: length=%d\n",
+		buffer->length);
 #endif
-	while (result && rest_of_data > 0 && rest_of_responses > 0
+	while (rest_of_data > 0 && rest_of_responses > 0
 	       && (buffer_fixed || interval-- > 0)
 	       && (eol_pos = memchr(cur_pos, '\n', rest_of_data)) != NULL)
 	  {
@@ -692,29 +762,26 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	    char *title_pos;
 
 #if DEBUG_DAT_MOST
-	    fprintf(stderr, "i=%d, offset=%d, rest_of_data=%d, cur_pos=0x%x\n",
-		    i, offset, rest_of_data, (int)cur_pos);
+	    fprintf(stderr, "i=%d, offset=%d, rest_of_data=%d, cur_pos=%p\n",
+		    i, offset, rest_of_data, cur_pos);
 #endif
 
 	    title_pos = extract_response(cur_pos, eol_pos, &response);
 
 #if DEBUG_DAT_MOST
-	    fprintf(stderr, "title_pos=0x%x\n", (int)title_pos);
+	    fprintf(stderr, "title_pos=%p\n", title_pos);
 #endif
 
 	    if (i == 0 && start == 0 && title_pos != NULL)
 	      {
-		char *title = g_strndup(title_pos, eol_pos - title_pos);
+		char *title = G_STRNDUP(title_pos, eol_pos - title_pos);
 		if (start_thread_cb != NULL)
 		  result = (*start_thread_cb)(thread, title, callback_data);
-		g_free(title);
+		G_FREE(title);
 		if (!result)
 		  break;
 	      }
 	    responses[i] = offset;
-	    cur_pos = eol_pos + 1;
-	    offset = cur_pos - buffer_top;
-	    rest_of_data = length - offset;
 
 	    if (title_pos != NULL)
 	      {
@@ -729,52 +796,68 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 		if (!result)
 		  break;
 	      }
+
+	    offset = (eol_pos + 1) - buffer_top;
+	    buffer_top = (char *)buffer->buffer;
+	    cur_pos = buffer_top + offset;
+	    rest_of_data = buffer->length - offset;
+
 #if DEBUG_DAT
-	    else
+	    if (title_pos == NULL)
 	      {
 		char *line;
 		fprintf(stderr, "response #%d: Couldn't analyze DAT file...ignoring parts.\n", i);
-		line = g_strndup(buffer_top + responses[i], offset - responses[i]);
+		line = G_STRNDUP(buffer_top + responses[i], offset - responses[i]);
 		fprintf(stderr, "failed: \"%s\"\n", line);
-		free(line);
+		G_FREE(line);
 		fprintf(stderr, "offset=%d\n", responses[i]);
 	      }
 #endif
+
 	    if (response.name != NULL)
-	      g_free((gpointer)response.name);
+	      G_FREE((gpointer)response.name);
 	    if (response.mailto != NULL)
-	      g_free((gpointer)response.mailto);
+	      G_FREE((gpointer)response.mailto);
 	    if (response.date_id != NULL)
-	      g_free((gpointer)response.date_id);
+	      G_FREE((gpointer)response.date_id);
 	    if (response.content != NULL)
-	      g_free((gpointer)response.content);
+	      G_FREE((gpointer)response.content);
 	  }
 
 	if (!buffer_fixed && buffer->fixed)
 	  {
 	    buffer_fixed = TRUE;
+	    eol_pos = NULL;
 	    continue;
 	  }
+
 	if (buffer_fixed)
 	  break;
 
-	if (!ochusha_async_buffer_wait(buffer, "ochusha_thread_2ch.c: ochusha_thread_2ch_parse_responses"))
-	  {
-	    /* bufferϤ޲٤callerˤޤ */
+	/*
+	 * XXX: ͥåȥ󥰥åɤĥäƤʤȤˤwaitʤ
+	 *      褦ˤ٤
+	 */
+	if (interval > 0
+	    || ochusha_async_buffer_is_busy(buffer,
+					    "ochusha_thread_2ch.c: ochusha_thread_2ch_parse_responses"))
+	  if (!ochusha_async_buffer_wait(buffer, "ochusha_thread_2ch.c: ochusha_thread_2ch_parse_responses"))
+	    {
+	      /* bufferϤ޲٤callerˤޤ */
 #if DEBUG_ASYNC_BUFFER_MOST
-	    fprintf(stderr, "ochusha_thread_2ch_parse_responses(): buffer has been terminated.\n");
+	      fprintf(stderr, "ochusha_thread_2ch_parse_responses(): buffer has been terminated.\n");
 #endif
-	    break;
-	  }
+	      break;
+	    }
 
-	if (((OchushaNetworkStatus *)buffer->user_data)->state
-	    == OCHUSHA_CACHE_IS_DIRTY)
+	if (status->state
+	    == OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY)
 	  {
 	    if (i > start && end_thread_cb != NULL)
 	      result = (*end_thread_cb)(thread, FALSE, callback_data);
 	    /* å夬äƤ뤳ȤϤ⤦狼ä */
-	    ((OchushaNetworkStatus *)buffer->user_data)->state
-	      = OCHUSHA_DIRECT_ACCESS;
+	    status->state
+	      = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_DIRECT_ACCESS;
 	    goto retry_analysis;
 	  }
       }
@@ -784,6 +867,13 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
   }
   ochusha_async_buffer_unlock(buffer);
 
+  if (status->last_modified != NULL)
+    {
+      if (thread_2ch->last_modified != NULL)
+	G_FREE(thread_2ch->last_modified);
+      thread_2ch->last_modified = G_STRDUP(status->last_modified);
+    }
+
   ochusha_async_buffer_active_unref(buffer, "ochusha_thread_2ch.c: ochusha_thread_2ch_parse_responses");
 
   if (end_thread_cb != NULL)
@@ -791,3 +881,43 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 
   return TRUE;
 }
+
+
+static const char *
+ochusha_thread_2ch_get_url_to_post(OchushaBBSThread *thread)
+{
+  OchushaThread2ch *thread_2ch;
+  char url[PATH_MAX];
+
+  g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread), NULL);
+
+  thread_2ch = OCHUSHA_THREAD_2CH(thread);
+  if (thread_2ch->post_url != NULL)
+    return thread_2ch->post_url;
+
+  snprintf(url, PATH_MAX, "%sl10",
+	   ochusha_thread_2ch_get_base_url(thread_2ch));
+  thread_2ch->post_url = G_STRDUP(url);
+  return thread_2ch->post_url;
+}
+
+
+static gboolean
+ochusha_thread_2ch_preview_response(OchushaBBSThread *thread,
+				    const OchushaBBSResponse *response,
+				    StartThreadCallback *start_cb,
+				    EachResponseCallback *response_cb,
+				    EndThreadCallback *end_cb,
+				    gpointer callback_data)
+{
+  return FALSE;
+}
+
+
+static gboolean
+ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
+				 OchushaConfig *config,
+				 const OchushaBBSResponse *response)
+{
+  return FALSE;
+}
