@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.h,v 1.1.1.1 2003/05/10 16:34:28 fuyu Exp $
+ * $Id: ochusha_thread_2ch.h,v 1.2 2003/05/25 08:46:16 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_THREAD_2CH_H_
@@ -56,14 +56,17 @@ struct _OchushaThread2ch
 {
   OchushaBBSThread parent_object;
 
-		      /* ʲDATե̾(id)board */
-  char *dat_url;      /* "http://pc.2ch.net/linux/dat/1022744633.dat" */
-  char *base_url;     /* "http://pc.2ch.net/test/read.cgi/linux/1022744633/" */
-  char *base_path;    /* "/test/read.cgi/linux/1022744633/" */
+		   /* ʲDATե̾(id)board */
+  char *dat_url;   /* "http://pc.2ch.net/linux/dat/1022744633.dat" */
+  char *base_url;  /* "http://pc.2ch.net/test/read.cgi/linux/1022744633/" */
+  char *base_path; /* "/test/read.cgi/linux/1022744633/" */
+  char *post_url;  /* "http://pc.2ch.net/test/read.cgi/linux/1022744633/l10" */
 
   /* responses[n]: nܤΥ쥹DAToffset */
   unsigned int *responses;
 
+  char *last_modified;	/* DATեɤ߹߻Last-Modifiedإå */
+
   /* ιŪ˻Ȥ */
   gboolean alive;
 };
